from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise

import datetime
import email.utils
import json
import os
import time
from six.moves.urllib.parse import urlparse


@implementer(interface.IPromise)
class RunPromise(GenericPromise):
  EXTENDED_STATUS_CODE_MAPPING = {
    '520': 'Too many redirects',
    '523': 'Connection error',
    '524': 'Connection timeout',
    '526': 'SSL Error',

  }

  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    # Set frequency compatible to default surykatka interval - 2 minutes
    self.setPeriodicity(float(self.getConfig('frequency', 2)))
    self.failure_amount = int(
      self.getConfig('failure-amount', self.getConfig('failure_amount', 1)))
    self.result_count = self.failure_amount
    self.error_list = []
    self.info_list = []
    # Make promise test-less, as it's result is not important for instantiation
    self.setTestLess()

  def appendErrorMessage(self, message):
    self.error_list.append(message)

  def appendInfoMessage(self, message):
    self.info_list.append(message)

  def emitLog(self):
   if len(self.error_list) > 0:
     emit = self.logger.error
   else:
     emit = self.logger.info

   message_list = self.error_list + self.info_list
   url = self.getConfig('url')
   if url:
     message_list.insert(0, '%s :' % (url,))
   emit(' '.join(message_list))

  def senseBotStatus(self):
    key = 'bot_status'

    def appendError(msg, *args):
      self.appendErrorMessage(key + ': ERROR ' + msg % args)

    if key not in self.surykatka_json:
      appendError("%r not in %r", key, self.json_file)
      return
    bot_status_list = self.surykatka_json[key]
    if len(bot_status_list) == 0:
      appendError("%r empty in %r", key, self.json_file)
      return
    bot_status = bot_status_list[0]
    if bot_status.get('text') != 'loop':
      appendError(
        "bot_status is %r instead of 'loop' in %r",
        str(bot_status.get('text')), self.json_file)
      return
    timetuple = email.utils.parsedate(bot_status['date'])
    last_bot_datetime = datetime.datetime.fromtimestamp(time.mktime(timetuple))
    last_bot_datetime_string = email.utils.formatdate(time.mktime(timetuple))
    delta = self.utcnow - last_bot_datetime
    # sanity check
    if delta < datetime.timedelta(minutes=0):
      appendError('Last bot datetime is in future')
      return
    if delta > datetime.timedelta(minutes=15):
      appendError('Last bot datetime is more than 15 minutes old')
      return

    self.appendInfoMessage('%s: OK Last bot status' % (key,))

  def senseSslCertificate(self):
    key = 'ssl_certificate'

    def appendError(msg, *args):
      self.appendErrorMessage(key + ': ERROR ' + msg % args)

    url = self.getConfig('url')
    parsed_url = urlparse(url)
    if parsed_url.scheme == 'https':
      hostname = parsed_url.netloc
      ssl_check = True
      certificate_expiration_days = self.getConfig(
        'certificate-expiration-days', '15')
      try:
        certificate_expiration_days = int(certificate_expiration_days)
      except ValueError:
        certificate_expiration_days = None
    else:
      ssl_check = False
      certificate_expiration_days = None
    if not ssl_check:
      return
    if certificate_expiration_days is None:
      appendError(
        'certificate-expiration-days %r is incorrect',
        self.getConfig('certificate-expiration-days'))
      return
    if not hostname:
      appendError('url is incorrect')
      return
    if key not in self.surykatka_json:
      appendError(
        'No key %r. If the error persist, please update surykatka.' % (key,))
      return
    entry_list = [
      q for q in self.surykatka_json[key] if q['hostname'] == hostname]
    if len(entry_list) == 0:
      appendError('No data')
      return
    for entry in entry_list:
      timetuple = email.utils.parsedate(entry['not_after'])
      if timetuple is None:
        appendError('No certificate information for %s' % (entry['ip']))
      else:
        certificate_expiration_time = datetime.datetime.fromtimestamp(
          time.mktime(timetuple))
        if certificate_expiration_time - datetime.timedelta(
          days=certificate_expiration_days) < self.utcnow:
          appendError(
            'Certificate on %s will expire on %s, which is less than %s days',
            entry['ip'], entry['not_after'], certificate_expiration_days)
          return
        else:
          self.appendInfoMessage(
            '%s: OK Certificate on %s will expire on %s, which is more than '
            '%s days' % (
              key, entry['ip'], entry['not_after'],
              certificate_expiration_days))
          return

  def senseHttpQuery(self):
    key = 'http_query'
    error = False

    def appendError(msg, *args):
      self.appendErrorMessage(key + ': ERROR ' + msg % args)

    if key not in self.surykatka_json:
      appendError("%r not in %r", key, self.json_file)
      return

    url = self.getConfig('url')
    status_code = self.getConfig('status-code')
    ip_list = self.getConfig('ip-list', '').split()
    http_header_dict = json.loads(self.getConfig('http-header-dict', '{}'))

    entry_list = [q for q in self.surykatka_json[key] if q['url'] == url]
    if len(entry_list) == 0:
      appendError('No data')
      return
    for entry in entry_list:
      entry_status_code = str(entry['status_code'])
      if entry_status_code != status_code:
        status_code_explanation = self.EXTENDED_STATUS_CODE_MAPPING.get(
          entry_status_code)
        if status_code_explanation:
          status_code_explanation = '%s (%s)' % (
            entry_status_code, status_code_explanation)
        else:
          status_code_explanation = entry_status_code
        appendError(
          'IP %s got status code %s instead of %s' % (
            entry['ip'], status_code_explanation, status_code))
        error = True
      if http_header_dict and http_header_dict != entry['http_header_dict']:
        appendError(
          'HTTP Header dict was %s instead of %s' % (
            json.dumps(entry['http_header_dict'], sort_keys=True),
            json.dumps(http_header_dict, sort_keys=True),
        ))
        error = True
    db_ip_list = [q['ip'] for q in entry_list]
    if len(ip_list):
      if set(ip_list) != set(db_ip_list):
        appendError(
          'expected IPs %s differes from got %s' % (
            ' '.join(ip_list), ' '.join(db_ip_list)))
        error = True
    if error:
      return
    info_message = '%s: OK with status code %s' % (key, status_code)
    if http_header_dict:
      info_message += ' and HTTP Header dict %s' % (
        json.dumps(http_header_dict, sort_keys=True),
      )
    if len(ip_list) > 0:
      info_message += ' on IPs %s' % (' '.join(ip_list))
    self.appendInfoMessage(info_message)

  def senseElapsedTime(self):
    key = 'elapsed_time'
    surykatka_key = 'http_query'

    def appendError(msg, *args):
      self.appendErrorMessage(key + ': ERROR ' + msg % args)

    if surykatka_key not in self.surykatka_json:
      appendError(
        'No key %r. If the error persist, please update surykatka.' % (
          surykatka_key,))
      return

    url = self.getConfig('url')
    maximum_elapsed_time = self.getConfig('maximum-elapsed-time')

    entry_list = [
      q for q in self.surykatka_json[surykatka_key] if q['url'] == url]
    if len(entry_list) == 0:
      appendError('No data')
      return
    for entry in entry_list:
      if maximum_elapsed_time:
        if 'total_seconds' in entry:
          maximum_elapsed_time = float(maximum_elapsed_time)
          if entry['total_seconds'] == 0.:
            appendError('IP %s failed to reply' % (entry['ip']))
          elif entry['total_seconds'] > maximum_elapsed_time:
            appendError(
              'IP %s replied in more time than maximum %.2fs' %
              (entry['ip'], maximum_elapsed_time))
          else:
            self.appendInfoMessage(
              '%s: OK IP %s replied in less time than maximum %.2fs' % (
                key, entry['ip'], maximum_elapsed_time))

  def sense(self):
    """
      Check if frontend URL is available
    """
    test_utcnow = self.getConfig('test-utcnow')
    if test_utcnow:
      self.utcnow = datetime.datetime.fromtimestamp(
        time.mktime(email.utils.parsedate(test_utcnow)))
    else:
      self.utcnow = datetime.datetime.utcnow()

    self.json_file = self.getConfig('json-file', '')
    if not os.path.exists(self.json_file):
      self.appendErrorMessage('ERROR File %r does not exists' % self.json_file)
    else:
      with open(self.json_file) as fh:
        try:
          self.surykatka_json = json.load(fh)
        except Exception:
          self.appendErrorMessage(
            "ERROR loading JSON from %r" % self.json_file)
        else:
          report = self.getConfig('report')
          if report == 'bot_status':
            self.senseBotStatus()
          elif report == 'http_query':
            self.senseHttpQuery()
            self.senseSslCertificate()
            self.senseElapsedTime()
          else:
            self.appendErrorMessage(
              "ERROR Report %r is not supported" % report)
    self.emitLog()

  def anomaly(self):
    return self._test(
      result_count=self.result_count, failure_amount=self.failure_amount)



from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise

import subprocess
import os
import psutil

@implementer(interface.IPromise)
class RunPromise(GenericPromise):

  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    # test load every 3 minutes
    self.setPeriodicity(minute=3)

  def checkCPULoad(self, tolerance=2.2):
    # tolerance=1.5 => accept CPU load up to 1.5 =150%
    load, load5, long_load = psutil.getloadavg()
    core_count = psutil.cpu_count()
    max_load = core_count * tolerance
    if long_load > max_load:
      # display top statistics
      top_result = subprocess.check_output(('top', '-n', '1', '-b'),
                                           universal_newlines=True)
      message = "CPU load is high: %s %s %s\n\n" % (load, load5, long_load)
      i = 0
      result_list = top_result.split('\n')
      # display first 5 lines
      while i < len(result_list) and i < 5:
        message += "\n%s" % result_list[i]
        i += 1
      self.logger.error(message)
    else:
      self.logger.info("CPU load is OK")

  def sense(self):
    load_threshold = self.getConfig('cpu-load-threshold')
    threshold = 0

    if load_threshold is not None:
      try:
        threshold = float(load_threshold)
      except ValueError as e:
        self.logger.error("CPU load threshold %r is not valid: %s" % (load_threshold, e))
        return

    self.checkCPULoad(threshold or 2.2)

  def test(self):
    # fail if load is higher than the threshold for more than 30 minutes
    return self._test(result_count=10, failure_amount=10)

  def anomaly(self):
    # fail if load is higher than the threshold for more than 30 minutes
    return self._test(result_count=10, failure_amount=10)



from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise

import subprocess


@implementer(interface.IPromise)
class RunPromise(GenericPromise):
  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    # SR can set custom periodicity
    self.setPeriodicity(float(self.getConfig('frequency', 2)))
    self.result_count = int(self.getConfig('result_count', '1'))
    self.failure_amount = int(self.getConfig('failure_amount', '1'))

  def sense(self):
    """
      Check result of the executed command
    """

    command = self.getConfig('command')

    try:
      out = subprocess.check_output(
        command,
        shell=True,
        stderr=subprocess.STDOUT)
      status = 0
    except subprocess.CalledProcessError as e:
      out = e.output
      status = e.returncode
    except Exception as e:
      self.logger.error(
        "ERROR %r during running command %r" % (e, command))
      return

    if status != 0:
      self.logger.error(
        'ERROR %r run with failure, output: %r' % (command, out))
    else:
      self.logger.info("OK %r run with success" % (command,))

  def anomaly(self):
    return self._anomaly(result_count=self.result_count, failure_amount=self.failure_amount)



##############################################################################
#
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.test.promise.plugin import TestPromisePluginMixin
import os
import time
from slapos.grid.promise import PromiseError


class TestCheckErrorOnZopeLongrequestLog(TestPromisePluginMixin):

  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    self.promise_name = "check-error-on-zope_longrequest-log.py"
    self.log_file = self.base_path + "/longrequest_logger_zope.log"
    self.test_log_file = self.base_path + "/SOFTINST-0_longrequest_logger_zope.log"
    self._update_logs()

  def get_time(self, sec):
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()-sec))

  def _update_logs(self):
    i = 600
    new = ""
    old = ""
    with open(self.log_file) as f:
      for line in f:
        new += line.replace("DATETIME", self.get_time(i))
        old += line.replace("DATETIME", self.get_time(i + 3600))
        i -= 1
      with open(self.test_log_file, "w") as f:
        f.write(old)
        f.write(new) 

  def test_01_no_delay_no_error_threshold(self):
    content = """from slapos.promise.plugin.check_error_on_zope_longrequest_log import RunPromise

extra_config_dict = {
  'log-file': '%(log_file)s',
  'error-threshold': 0,
  'maximum-delay': 0
}
""" % {'log_file': self.test_log_file}
    self.writePromise(self.promise_name, content)

    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['message'], "ERROR: Site has 6 long request")

  def test_02_no_delay_error_threshold(self):
    content = """from slapos.promise.plugin.check_error_on_zope_longrequest_log import RunPromise

extra_config_dict = {
  'log-file': '%(log_file)s',
  'error-threshold': 7,
  'maximum-delay': 0
}
""" % {'log_file': self.test_log_file}
    self.writePromise(self.promise_name, content)

    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['message'], "INFO: Site has 6 long request")

  def test_03_delay_no_error_threshold(self):
    content = """from slapos.promise.plugin.check_error_on_zope_longrequest_log import RunPromise

extra_config_dict = {
  'log-file': '%(log_file)s',
  'error-threshold': 2,
  'maximum-delay': 3600
}
""" % {'log_file': self.test_log_file}
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['message'], "ERROR: Site has 3 long request")


if __name__ == '__main__':
  unittest.main()



# coding: utf-8
# Copyright (C) 2021  Nexedi SA and Contributors.
#                     Łukasz Nowak <luke@nexedi.com>
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.

import dns.resolver
import json
import mock
import os
import shutil
import tempfile
import unittest

from slapos import dnsresolver


class MockAnswer(object):
    def __init__(self, address):
        self.address = address


class Answer(object):
  def __init__(self, address):
    self.address = address


class DNSResolverTestCase(unittest.TestCase):
  def _query(self, h):
    return self._query_answer.get(h, [])

  def setUp(self):
    self.working_directory = tempfile.mkdtemp()
    self.patcher_list = [
      mock.patch.object(dns.resolver.Resolver, "query", new=self._query)
    ]
    [q.start() for q in self.patcher_list]

  def tearDown(self):
    shutil.rmtree(self.working_directory, True)
    [q.stop() for q in self.patcher_list]

  def _fillInput(self, value):
    with tempfile.NamedTemporaryFile(
      dir=self.working_directory,
      delete=False) as fh:
      fh.write(value.encode())
      return fh.name

  def test(self):
    ip = '123.124.125.126'
    input_list = [
      self._fillInput('1.example.com non.example.com\n%s' % (ip,)),
      self._fillInput('2.example.com %s' % (ip,)),
      self._fillInput(''),
      self._fillInput('# 1.2.3.4'),
    ]
    output = os.path.join(self.working_directory, 'output')

    self._query_answer = {
     '1.example.com': [Answer('127.0.0.1'), Answer('127.0.0.3')],
     '2.example.com': [Answer('127.0.0.2')]
    }

    dnsresolver.app("list", output, input_list, -1)

    self.assertTrue(os.path.exists(output))

    with open(output, 'rb') as fh:
      self.assertEqual(
        json.load(fh),
        ['123.124.125.126', '127.0.0.1', '127.0.0.2', '127.0.0.3']
      )



from __future__ import print_function

import datetime
import json
import sys
import traceback
import time
import feedparser
from uritemplate import expand

import slapos.slap
from slapos.slap import SoftwareProductCollection

from slapos.slap.slap import ConnectionError
from requests.exceptions import HTTPError


from erp5.util.taskdistribution import SAFE_RPC_EXCEPTION_LIST

SOFTWARE_PRODUCT_NAMESPACE = "product."

SOFTWARE_STATE_UNKNOWN = "SOFTWARE_STATE_UNKNOWN"
SOFTWARE_STATE_INSTALLING = "SOFTWARE_STATE_INSTALLING"
SOFTWARE_STATE_INSTALLED = "SOFTWARE_STATE_INSTALLED"
SOFTWARE_STATE_DESTROYING = "SOFTWARE_STATE_DESTROYING"

INSTANCE_STATE_UNKNOWN = "INSTANCE_STATE_UNKNOWN"
INSTANCE_STATE_STARTING = "INSTANCE_STATE_STARTING"
INSTANCE_STATE_STARTED = "INSTANCE_STATE_STARTED"
INSTANCE_STATE_STARTED_WITH_ERROR = "INSTANCE_STATE_STARTED_WITH_ERROR"
INSTANCE_STATE_STOPPING = "INSTANCE_STATE_STOPPING"
INSTANCE_STATE_STOPPED = "INSTANCE_STATE_STOPPED"
INSTANCE_STATE_DESTROYING = "INSTANCE_STATE_DESTROYING"

TESTER_STATE_INITIAL = "TESTER_STATE_INITIAL"
TESTER_STATE_NOTHING = "TESTER_STATE_NOTHING"
TESTER_STATE_SOFTWARE_INSTALLED = "TESTER_STATE_SOFTWARE_INSTALLED"
TESTER_STATE_INSTANCE_INSTALLED = "TESTER_STATE_INSTANCE_INSTALLED"
TESTER_STATE_INSTANCE_STARTED = "TESTER_STATE_INSTANCE_STARTED"
TESTER_STATE_INSTANCE_UNINSTALLED = "TESTER_STATE_INSTANCE_UNINSTALLED"

class TestTimeout(Exception):
  pass



# Simple decorator to prevent raise due small
# network failures.
def retryOnNetworkFailure(func):
  def wrapper(*args, **kwargs):
    retry_time = 64
    while True:
      try:
        return func(*args, **kwargs)
      except SAFE_RPC_EXCEPTION_LIST as e:
        print('Network failure: %s , %s' % (sys.exc_info(), e))
      except HTTPError as e:
        print('Network failure: %s , %s' % (sys.exc_info(), e))
      except ConnectionError as e:
        print('Network failure: %s , %s' % (sys.exc_info(), e))
      except slapos.slap.ConnectionError as e:
        print('Network failure: %s , %s' % (sys.exc_info(), e))

      print('Retry method %s in %i seconds' % (func, retry_time))
      time.sleep(retry_time)
      retry_time = min(retry_time*1.5, 640)


  wrapper.__name__ = func.__name__
  wrapper.__doc__ = func.__doc__
  return wrapper


class SlapOSMasterCommunicator(object):


  def __init__(self, slap, slap_supply, slap_order, url, logger):

    self._logger = logger
    self.slap = slap
    self.slap_order = slap_order
    self.slap_supply = slap_supply
    self.hateoas_navigator = self.slap._hateoas_navigator
    self.hosting_subscription_url = None

    if url is not None and \
      url.startswith(SOFTWARE_PRODUCT_NAMESPACE):

      product = SoftwareProductCollection(self._logger, self.slap)
      try:
        url = product.__getattr__(url[len(SOFTWARE_PRODUCT_NAMESPACE):])
      except AttributeError as e:
        self._logger.warning('Error on get software release : %s ' % e.message)

    self.url = url  

  @retryOnNetworkFailure
  def _supply(self, state):
    if self.computer_guid is None:
      self._logger.info('Nothing to supply for %s.' % (self.name))
      return None

    self._logger.info('Supply %s@%s: %s', self.url, self.computer_guid,
        state)
    return self.slap_supply.supply(self.url, self.computer_guid, state)

  @retryOnNetworkFailure
  def _request(self, state):
    self._logger.info('Request %s@%s: %s', self.url, self.name, state)
    self.latest_state = state
    return self.slap_order.request(
          software_release=self.url,
          partition_reference=self.name,
          state=state,
          **self.request_kw)

  @retryOnNetworkFailure
  def _hateoas_getComputer(self, reference):

    root_document = self.hateoas_navigator.getRootDocument()
    search_url = root_document["_links"]['raw_search']['href']

    getter_link = expand(search_url, { 
      "query": "reference:%s AND portal_type:Computer" % reference, 
      "select_list": ["relative_url"],
      "limit": 1})

    result = self.hateoas_navigator.GET(getter_link)
    content_list = json.loads(result)['_embedded']['contents']

    if len(content_list) == 0:
      raise Exception('No Computer found.')

    computer_relative_url = content_list[0]["relative_url"]
    
    getter_url = self.hateoas_navigator.getDocumentAndHateoas(
      computer_relative_url)

    return json.loads(self.hateoas_navigator.GET(getter_url))
    
 
  @retryOnNetworkFailure
  def getSoftwareInstallationList(self):
    # XXX Move me to slap.py API 

    computer = self._hateoas_getComputer(self.computer_guid)

    # Not a list ?
    action = computer['_links']['action_object_slap']

    if action.get('title') == 'getHateoasSoftwareInstallationList':
      getter_link = action['href']
    else:
      raise Exception('No Link found found.')

    result = self.hateoas_navigator.GET(getter_link)
    return json.loads(result)['_links']['content']


  @retryOnNetworkFailure
  def getSoftwareInstallationNews(self):
    getter_link = None
    for si in self.getSoftwareInstallationList():
      if si["title"] == self.url:
        getter_link = si["href"]
        break

    # We could not find the document, so it is probably too soon.
    if getter_link is None:
      return ""
    
    result = self.hateoas_navigator.GET(getter_link)
    action_object_slap_list = json.loads(result)['_links']['action_object_slap']

    for action in action_object_slap_list:
      if action.get('title') == 'getHateoasNews':
        getter_link = action['href']
        break
    else:
      raise Exception('getHateoasNews not found.')

    result = self.hateoas_navigator.GET(getter_link)
    if len(json.loads(result)['news']) > 0:
      return json.loads(result)['news'][0]["text"]
    return ""

  @retryOnNetworkFailure
  def getInstanceUrlList(self):

    if self.hosting_subscription_url is None:
      hosting_subscription_dict = self.hateoas_navigator._hateoas_getHostingSubscriptionDict()
      for hs in hosting_subscription_dict:
        if hs['title'] == self.name:
          self.hosting_subscription_url = hs['href'] 
          break 

    if self.hosting_subscription_url is None:
      return None

    return self.hateoas_navigator.getHateoasInstanceList(
            self.hosting_subscription_url)

  @retryOnNetworkFailure
  def getNewsFromInstance(self, url):

    result = self.hateoas_navigator.GET(url)
    result = json.loads(result)
    if result['_links'].get('action_object_slap', None) is None:
      return None

    object_link = self.hateoas_navigator.hateoasGetLinkFromLinks(
       result['_links']['action_object_slap'], 'getHateoasNews')
    
    result = self.hateoas_navigator.GET(object_link)
    return json.loads(result)['news']

  @retryOnNetworkFailure
  def getInformationFromInstance(self, url):

    result = self.hateoas_navigator.GET(url)
    result = json.loads(result)
    if result['_links'].get('action_object_slap', None) is None:
      print(result['links'])
      return None

    object_link = self.hateoas_navigator.hateoasGetLinkFromLinks(
       result['_links']['action_object_slap'], 'getHateoasInformation')

    result = self.hateoas_navigator.GET(object_link)
    return json.loads(result)


class SoftwareReleaseTester(SlapOSMasterCommunicator):
  deadline = None
  latest_state = None

  def __init__(self,
              name,
              logger,
              slap,
              slap_order,
              slap_supply,
              url, # software release url
              computer_guid=None, # computer for supply if desired
              request_kw=None, # instance parameters, if instantiation
                               # testing is desired
              software_timeout=3600,
              instance_timeout=3600,
          ):

    super(SoftwareReleaseTester, self).__init__( 
      slap, slap_supply, slap_order, url, logger) 

    self.name = name
    self.computer_guid = computer_guid

    if isinstance(request_kw, str) or \
      isinstance(request_kw, unicode):
      self.request_kw = json.loads(request_kw)
    else:
      self.request_kw = request_kw
    self.message_history = []
 
    self.state = TESTER_STATE_INITIAL
    self.transition_dict = {
      # step function
      # delay
      # next_state
      # software_state
      # instance_state
      TESTER_STATE_INITIAL: (
        lambda t: None,
        None,
        TESTER_STATE_NOTHING,
        None,
        None,
      ),
      TESTER_STATE_NOTHING: (
        lambda t: t._supply("available"),
        int(software_timeout),
        request_kw is None and TESTER_STATE_INSTANCE_UNINSTALLED or \
            TESTER_STATE_SOFTWARE_INSTALLED,
        SOFTWARE_STATE_INSTALLED,
        None,
      ),
      TESTER_STATE_SOFTWARE_INSTALLED: (
        lambda t: t._request("started"),
        int(instance_timeout),
        TESTER_STATE_INSTANCE_STARTED,
        None,
        INSTANCE_STATE_STARTED,
      ),
      TESTER_STATE_INSTANCE_STARTED: (
        lambda t: t._request("destroyed"),
        int(1200),
        TESTER_STATE_INSTANCE_UNINSTALLED,
        None,
        INSTANCE_STATE_STOPPED,
      ),
      TESTER_STATE_INSTANCE_UNINSTALLED: (
        lambda t: t._supply("destroyed"),
        int(1200),
        None,
        None,
        None,
      ),
     }

  def __repr__(self):
      deadline = self.deadline
      if deadline is not None:
          deadline -= time.time()
          deadline = '+%is' % (deadline, )
      return '<%s(state=%s, deadline=%s) at %x>' % (
          self.__class__.__name__, self.state, deadline, id(self))

  def getInfo(self):
    info = ""
    info += "Software Release URL: %s\n" % (self.url)
    if self.computer_guid is not None:
      info += "Supply requested on: %s\n" % (self.computer_guid)
    info += "Instance Requested (Parameters): %s\n" % self.request_kw
    return info

  def getFormatedLastMessage(self):
    if len(self.message_history) == 0:
      return "No message"

    summary = "Summary about the test. Instance List and Status:\n"
    message = "Last information about the tester:\n"
    if self.message_history[-1] is not None:
      message_list = self.message_history[-1]
      for entry in message_list:
        summary += "%s %s -> %s\n" % (
          entry['title'], entry["slave"] and "(slave)" or "", entry['state'])
        for prop in entry:
          if prop != "information":
            message += "%s = %s\n" % (prop, json.dumps(entry[prop], indent=2))
          
        message += "=== connection_dict === \n%s\n" % (
          json.dumps(entry["information"]["connection_dict"], indent=2))
        message += "\n"
        message += "=== parameter_dict === \n%s\n" % (
          json.dumps(entry["information"]["parameter_dict"], indent=2))
        message += "\n"
      message += "="*79
      message += "\n\n\n"
 
    return summary + message

  def _getSoftwareState(self):
    if self.computer_guid is None:
      return SOFTWARE_STATE_INSTALLED

    message = self.getSoftwareInstallationNews()
    if message.startswith("#error no data found"):
      return SOFTWARE_STATE_UNKNOWN

    if message.startswith('#access software release'):
      return SOFTWARE_STATE_INSTALLED

    if message.startswith('#error'):
      return SOFTWARE_STATE_INSTALLING

    return SOFTWARE_STATE_UNKNOWN 

  @retryOnNetworkFailure
  def getRSSEntryFromMonitoring(self, base_url):
    if base_url is None:
      return {}

    feed_url = base_url + '/monitor-public/rssfeed.html'
    d = feedparser.parse(feed_url) 
    
    if len(d.entries) > 0:
      return {"date": d.entries[0].published, 
              "message": d.entries[0].description, 
              "title" : d.entries[0].title}
        
    return {}

  @retryOnNetworkFailure
  def _getInstanceState(self):
    latest_state = self.latest_state
    self._logger.debug('latest_state = %r', latest_state)

    if latest_state is None:
      return INSTANCE_STATE_UNKNOWN

    message_list = [] 
    try:
      for instance in self.getInstanceUrlList():
        news = self.getNewsFromInstance(instance["href"])
        information = self.getInformationFromInstance(instance["href"])
        state = INSTANCE_STATE_UNKNOWN
        monitor_information_dict = {}

        info_created_at = "-1"
        is_slave = information['slave']
        if is_slave:
          self._logger.debug('Instance is slave')
          if (information["connection_dict"]) > 0:
            state =  INSTANCE_STATE_STARTED
        else:
          # not slave
          instance_state = news[0]
          if instance_state.get('created_at', '-1') != "-1":
            # the following does NOT take TZ into account
            created_at = datetime.datetime.strptime(instance_state['created_at'], 
              '%a, %d %b %Y %H:%M:%S %Z')
            gmt_now = datetime.datetime(*time.gmtime()[:6])

            info_created_at = '%s (%d)' % (
               instance_state['created_at'], (gmt_now - created_at).seconds)

            if instance_state['text'].startswith('#access'):
              state =  INSTANCE_STATE_STARTED

            if instance_state['text'].startswith('#access Instance correctly stopped'):
              state =  INSTANCE_STATE_STOPPED

            if instance_state['text'].startswith('#error'):
              state = INSTANCE_STATE_STARTED_WITH_ERROR

        if state == INSTANCE_STATE_STARTED_WITH_ERROR:
          # search for monitor url
          monitor_v6_url = information["connection_dict"].get("monitor_v6_url")
          try:
            monitor_information_dict = self.getRSSEntryFromMonitoring(monitor_v6_url)
          except Exception:
            self._logger.exception('Unable to download promises for: %s' % (instance["title"]))
            self._logger.info(traceback.format_exc())
            monitor_information_dict = {"message": "Unable to download"}

        self._logger.info('Instance state: %s -> %s' % (instance['title'], state))
        self._logger.info('Instance Created at: %s -> %s' % (instance['title'], info_created_at))
 
        message_list.append({
          'title': instance["title"],
          'slave': is_slave,
          'news': news[0],
          'information': information,
          'monitor': monitor_information_dict,
          'state': state
        })

    except slapos.slap.ServerError:
      self._logger.exception('Got an error requesting partition for '
            'its state')
      return INSTANCE_STATE_UNKNOWN
 
    started = 0
    stopped = 0
    self.message_history.append(message_list)
    for instance in message_list:
      if not instance['slave'] and \
        instance['state'] in (INSTANCE_STATE_UNKNOWN, INSTANCE_STATE_STARTED_WITH_ERROR):
        return instance['state']
      elif not instance['slave'] and instance['state'] == INSTANCE_STATE_STARTED:
        started = 1
      elif not instance['slave'] and instance['state'] == INSTANCE_STATE_STOPPED:
        stopped = 1
      
      if instance['slave'] and instance['state'] == INSTANCE_STATE_UNKNOWN:
        return instance['state']

    if started and stopped:
      return INSTANCE_STATE_UNKNOWN
    
    if started:
      return INSTANCE_STATE_STARTED

    if stopped:
      return INSTANCE_STATE_STOPPED

  @retryOnNetworkFailure
  def teardown(self):
    """
    Interrupt a running test sequence, putting it in idle state.
    """
    self._logger.info('Invoking TearDown for %s@%s' % (self.url, self.name))
    if self.request_kw is not None:
       self._request('destroyed')
    if self.computer_guid is not None:
      self._supply('destroyed')
    self.state = TESTER_STATE_INSTANCE_UNINSTALLED

  def tic(self, now):
    """
    Check for missed deadlines (-> test failure), conditions for moving to
    next state, and actually moving to next state (executing its payload).
    """
    self._logger.debug('TIC')
    deadline = self.deadline

    if deadline < now and deadline is not None:
      raise TestTimeout(self.state)

    _, _, next_state, software_state, instance_state = self.transition_dict[
        self.state]

    if (software_state is None or
          software_state == self._getSoftwareState()) and (
          instance_state is None or
          instance_state == self._getInstanceState()):

      self._logger.debug('Going to state %s (%r)', next_state, instance_state)
      if next_state is None:
        return None

      self.state = next_state
      stepfunc, delay, _, _, _ = self.transition_dict[next_state]
      self.deadline = now + delay
      stepfunc(self)
    return self.deadline




# -*- coding: utf-8 -*-
##############################################################################
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly advised to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
##############################################################################

import mock
import os
import six
from slapos.grid.promise import PromiseError
from slapos.promise.plugin.check_neo_health import RunPromise
from . import TestPromisePluginMixin


class TestCheckNeoHealth(TestPromisePluginMixin):

  promise_name = "monitor-neo-health.py"

  def setUp(self):
    super(TestCheckNeoHealth, self).setUp()
    self.writePromise(neoctl='neoctl')

  def writePromise(self, **kw):
    super(TestCheckNeoHealth, self).writePromise(self.promise_name,
      "from %s import %s\nextra_config_dict = %r\n"
      % (RunPromise.__module__, RunPromise.__name__, kw))

  def runPromise(self, summary, failed=False):
    self.configureLauncher(enable_anomaly=True)
    with mock.patch('subprocess.check_output', return_value=summary):
      if failed:
        self.assertRaises(PromiseError, self.launcher.run)
      else:
        self.launcher.run()
    result = self.getPromiseResult(self.promise_name)['result']
    self.assertEqual(result['failed'], failed)
    return result['message']

  def test_running(self):
    message = "\
RUNNING; UP_TO_DATE=12; ltid=03d0a8a41ea17cb9 (2019-06-26 19:48:07.179095)"
    self.assertEqual(message, self.runPromise("# {}\n%s\n" % message))

  def test_recovering(self):
    message = self.runPromise("""# {"problem": [null]}
RECOVERING; UP_TO_DATE=12; DOWN=1
""", True)
    self.assertEqual(message, "PROBLEM (main)")

  def test_lag(self):
    message = self.runPromise("""# {"warning": ["neo_21"]}
RUNNING; UP_TO_DATE=1; ltid=03d17cd35613ac88 (2019-08-02 13:07:20.174262)

neo_21
    BACKINGUP; UP_TO_DATE=1; ltid=03d17cd35613ac87; lag=ε
""", True)
    self.assertEqual(message, "WARNING (backup: neo_21)")

  def test_out_of_date(self):
    message = self.runPromise("""# {"warning": [null]}
BACKINGUP; OUT_OF_DATE=1, UP_TO_DATE=3; ltid=03d17cd516db69db (2019-08-02 13:09:05.357129)
""", True)
    self.assertEqual(message, "WARNING (main)")

  def test_critical(self):
    message = self.runPromise("ERROR: not connected", True)
    if six.PY3:
      self.assertEqual(message, "Expecting value: line 1 column 1 (char 0)")
      return
    self.assertEqual(message, "No JSON object could be decoded")


if __name__ == '__main__':
  unittest.main()



# coding: utf-8
# Copyright (C) 2021  Nexedi SA and Contributors.
#                     Łukasz Nowak <luke@nexedi.com>
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.
from __future__ import print_function

import click
import dns.exception
import dns.resolver
import ipaddress
import json
import os
import time

TIMEOUT = 5


@click.command(short_help="Resolves domains and IPv4s to IPv4s")
@click.option(
  "--style",
  "-s",
  help="Output style of the JSON.",
  type=click.Choice(["list", "full"]),
  default="list",
  show_default=True
)
@click.option(
  "--output",
  "-o",
  help="Output file.",
  type=click.Path()  # can't use click.File, as file has to be opened an
                     # closed on each run
)
@click.option(
  "--delay",
  "-d",
  help="Delay in seconds between iterations, use -1 for one time processing.",
  default=300,
  show_default=True,
  type=click.INT
)
@click.argument(
  'input_list',
  nargs=-1,
  type=click.Path(),  # can't use click.File, as existence is required and
                      # that increases usage complexity
)
def cli(
  style,
  output,
  input_list,
  delay
):
  app(style, output, input_list, delay)


def app(style, output, input_list, delay):
  while True:
    address_dict = {}
    # use default resolver of the machine, as this is expected approach
    resolver = dns.resolver.Resolver(configure=True)
    resolver.timeout = TIMEOUT
    resolver.lifetime = TIMEOUT
    resolver.edns = -1
    for filename in input_list:
      if not os.path.exists(filename):
        continue
      with open(filename, 'rb') as fh:
        for line in fh.readlines():
          line = line.decode('utf-8').strip()
          if line.startswith('#'):
            continue
          for entry in line.split():
            try:
              address_dict[entry] = [
                str(ipaddress.ip_address(entry).exploded)]
            except Exception:
              try:
                address_dict[entry] = [
                  answer.address for answer in resolver.query(entry)]
              except (
                  dns.resolver.NXDOMAIN,
                  dns.resolver.NoAnswer,
                  dns.exception.Timeout,
                  dns.resolver.NoNameservers,
              ):
                address_dict[entry] = []
    if style == 'list':
      merged_list = []
      for q in address_dict.values():
        merged_list.extend(q)
      dumps = json.dumps(sorted(set(merged_list)), indent=2)
    else:
      dumps = json.dumps(address_dict, indent=2)

    if output is None:
      print(dumps)
    else:
      with open(output, 'wb') as fh:
        fh.write(dumps.encode())
    if delay == -1:
      break
    else:
      time.sleep(delay)


if __name__ == '__main__':
  cli()



import json
import os
import random
import shutil
import string
import sys
import tempfile
import textwrap
import unittest

import mock

import slapos.runner.utils as runner_utils
import slapos.runner.views
import supervisor
sys.modules['slapos.runner.utils'].sup_process = mock.MagicMock()


class TestRunnerBackEnd(unittest.TestCase):
  def setUp(self):
    self.sup_process = runner_utils.sup_process
    self.sup_process.reset_mock()
    runner_utils.open = open

  def tearDown(self):
    garbage_file_list = [
      os.path.join(*(os.getcwd(), '.htpasswd')),
      os.path.join(*(os.getcwd(), '.turn-left')),
      os.path.join(*(os.getcwd(), 'slapos-test.cfg')),
    ]
    for garbage_file in garbage_file_list:
      if os.path.exists(garbage_file):
        os.remove(garbage_file)

  def test_UserCanLoginAndUpdateCredentials(self):
    """
    * Create a user with createNewUser
    * Tests user can login with checkUserCredential
    * Updates user password updateUserCredential
    * Checks user can login with new credentials
    """
    def generate_password():
      return "".join(random.sample( \
        string.ascii_letters + string.digits + string.punctuation, 20))

    config = {'etc_dir': os.getcwd()}
    login = "admin"
    password = generate_password()
    self.assertTrue(runner_utils.createNewUser(config, login, password))
    self.assertTrue(runner_utils.checkUserCredential(config, login, password))

    new_password = generate_password()
    self.assertNotEqual(password, new_password)
    runner_utils.updateUserCredential(config, login, new_password)
    self.assertTrue(runner_utils.checkUserCredential(config, login, new_password))

  @mock.patch('os.path.exists')
  def test_getCurrentSoftwareReleaseProfile(self, mock_path_exists):
    """
    * Mock a .project file
    * Tests that getCurrentSoftwareReleaseProfile returns an absolute path
    """
    cwd = os.getcwd()

    # If .project file doesn't exist, then getCurrentSoftwareReleaseProfile
    # returns an empty string
    config = {'etc_dir': os.path.join(cwd, 'etc'),
              'workspace': os.path.join(cwd, 'srv', 'runner'),
              'software_profile': 'software.cfg'}

    profile = runner_utils.getCurrentSoftwareReleaseProfile(config)
    self.assertEqual(profile, "")

    # If .project points to a SR that doesn't exist, returns empty string
    runner_utils.open = mock.mock_open(read_data="workspace/fake/path/")
    mock_path_exists.return_value = False
    profile = runner_utils.getCurrentSoftwareReleaseProfile(config)
    self.assertEqual(profile, "")

    # If software_profile exists, getCurrentSoftwareReleaseProfile should
    # return its absolute path
    runner_utils.open = mock.mock_open(read_data = "workspace/project/software/")
    mock_path_exists.return_value = True
    profile = runner_utils.getCurrentSoftwareReleaseProfile(config)
    self.assertEqual(profile, os.path.join(config['workspace'], 'project',
        'software', config['software_profile']))

  @mock.patch('os.mkdir')
  @mock.patch('slapos.runner.utils.updateProxy')
  @mock.patch('slapos.runner.utils.requestInstance')
  @mock.patch('slapos.runner.utils.config_SR_folder')
  def _runSlapgridWithLockMakesCorrectCallsToSupervisord(self,
                                                         run_slapgrid_function,
                                                         process_name,
                                                         mock_configSRFolder,
                                                         mock_requestInstance,
                                                         mock_updateProxy,
                                                         mock_mkdir):
    """
    Tests that runSoftwareWithLock and runInstanceWithLock make correct calls
    to sup_process (= supervisord)
    """
    mock_updateProxy.return_value = True
    cwd = os.getcwd()
    config = {'etc_dir': cwd,
              'software_root': os.path.join(cwd, 'software'),
              'software_log': os.path.join(cwd, 'software.log'),
              'instance_root': os.path.join(cwd, 'software'),
              'instance_log': os.path.join(cwd, 'software.log')}
    # If process is already running, then does nothing
    self.sup_process.isRunning.return_value = True
    self.assertEqual(run_slapgrid_function(config), 1)
    self.assertFalse(self.sup_process.runProcess.called)

    # If the slapgrid process is not running, it should start it
    self.sup_process.isRunning.return_value = False
    # First, without Lock
    run_slapgrid_function(config)
    self.sup_process.runProcess.assert_called_once_with(config, process_name)
    self.assertFalse(self.sup_process.waitForProcessEnd.called)
    # Second, with Lock
    self.sup_process.reset_mock()
    run_slapgrid_function(config, lock=True)
    self.sup_process.runProcess.assert_called_once_with(config, process_name)
    self.sup_process.waitForProcessEnd.assert_called_once_with(config, process_name)

  def test_runSoftwareWithLockMakesCorrectCallstoSupervisord(self):
    self._runSlapgridWithLockMakesCorrectCallsToSupervisord(
      runner_utils.runSoftwareWithLock, 'slapgrid-sr')

  def test_runInstanceWithLockMakesCorrectCallstoSupervisord(self):
    self._runSlapgridWithLockMakesCorrectCallsToSupervisord(
      runner_utils.runInstanceWithLock, 'slapgrid-cp')

  @mock.patch('os.path.exists')
  @mock.patch('os.remove')
  @mock.patch('slapos.runner.utils.startProxy')
  @mock.patch('slapos.runner.utils.stopProxy')
  @mock.patch('slapos.runner.utils.removeProxyDb')
  def test_changingSRUpdatesProjectFileWithExistingPath(self,
                                                        mock_removeProxyDb,
                                                        mock_stopProxy,
                                                        mock_startProxy,
                                                        mock_remove,
                                                        mock_path_exists):
    cwd = os.getcwd()
    config = {'etc_dir' : os.path.join(cwd, 'etc'),
              'workspace': os.path.join(cwd, 'srv', 'runner')}
    projectpath = 'workspace/project/software/'
    self.assertNotEqual(runner_utils.realpath(config, projectpath, \
                                              check_exist=False), '')

    # If projectpath doesn't exist, .project file shouldn't be written
    mock_path_exists.return_value = False
    result = runner_utils.configNewSR(config, projectpath)
    self.assertFalse(result)

    # If projectpath exist, .project file should be overwritten
    mock_path_exists.return_value = True
    runner_utils.open = mock.mock_open()
    result = runner_utils.configNewSR(config, projectpath)
    self.assertTrue(result)
    runner_utils.open.assert_has_calls([mock.call().write(projectpath)])

  @mock.patch('slapos.runner.utils.isInstanceRunning')
  @mock.patch('slapos.runner.utils.svcStopAll')
  def test_removingInstanceStopsProcessesAndCleansInstanceDirectory(self,
                                                                    mock_svcStopAll,
                                                                    mock_isInstanceRunning):
    """
    When removing the current running instances, processes should be stopped
    and directories deleted properly
    """
    cwd = os.getcwd()
    config = {'database_uri': os.path.join(cwd, 'proxy.db'),
              'etc_dir': os.path.join(cwd, 'etc'),
              'instance_root': os.path.join(cwd, 'instance'),}

    # If slapos node is running, removeCurrentInstance returns a string
    mock_isInstanceRunning.return_value = True
    self.assertTrue(isinstance(runner_utils.removeCurrentInstance(config), str))
    self.assertTrue(mock_isInstanceRunning.called)

    # If slapos is not running, process should be stopped and directories emptied
    mock_isInstanceRunning.return_value = False
    result = runner_utils.removeCurrentInstance(config)
    self.assertTrue(mock_svcStopAll.called)
    self.sup_process.stopProcess.assert_called_with(config, 'slapproxy')

  @mock.patch('os.listdir')
  @mock.patch('os.path.exists')
  @mock.patch('slapos.runner.utils.removeCurrentInstance')
  @mock.patch('slapos.runner.utils.removeSoftwareRootDirectory')
  def test_removingUsedSoftwareReleaseCleansInstancesToo(self,
                                                         mock_removeSoftwareRootDirectory,
                                                         mock_removeCurrentInstance,
                                                         mock_path_exists,
                                                         mock_listdir):
    """
    When removing the Software Release on which depends the current running
    instances, the current instances should be stopped and removed properly.
    """
    # mock_listir is needed for not raising in loadSoftwareRList or future equivalent
    mock_listdir.return_value = []

    cwd = os.getcwd()
    config = {'etc_dir': os.path.join(cwd, 'etc'),
              'software_root': os.path.join(cwd, 'software'),
              'software_link': os.path.join(cwd, 'softwareLink'),}

    self.sup_process.isRunning.return_value = False

    # First tests that if the current instance doesn't extend the Software
    # Release to delete, the instance isn't deleted
    runner_utils.open = mock.mock_open(read_data="/workspace/my_project/software/another/")
    runner_utils.removeSoftwareByName(config, '1234567890', 'my_software_name')

    self.assertFalse(mock_removeCurrentInstance.called)
    self.assertTrue(mock_removeSoftwareRootDirectory.called)

    # If the current Instance extends the Software Release, then both must
    # be removed
    mock_removeSoftwareRootDirectory.reset_mock()

    runner_utils.open = mock.mock_open(read_data="/workspace/my_project/software/my_software_name/")
    runner_utils.removeSoftwareByName(config, '1234567890', 'my_software_name')

    self.assertTrue(mock_removeCurrentInstance.called)
    self.assertTrue(mock_removeSoftwareRootDirectory.called)

  @mock.patch('slapos.runner.utils.runInstanceWithLock')
  @mock.patch('slapos.runner.utils.runSoftwareWithLock')
  def test_runSoftwareRunOnlyOnceIfSoftwareSuccessfullyCompiledOnFirstTime(self,
                                                                           mock_runSoftwareWithLock,
                                                                           mock_runInstanceWithLock):
    cwd = os.getcwd()
    config = {'runner_workdir': cwd,
              'etc_dir': cwd}

    build_and_run_parameter_dict = {
      'run_instance': False,
      'run_software': True,
      'max_run_instance': 3,
      'max_run_software': 3,
    }
    runner_utils.saveBuildAndRunParams(config, build_and_run_parameter_dict)

    # First, configuration is set to only run the compilation of the software release
    # Both runSoftwareWithLock and runInstanceWithLock succeed on 1st try
    mock_runSoftwareWithLock.return_value = 0
    mock_runInstanceWithLock.return_value = 0

    runner_utils.runSlapgridUntilSuccess(config, 'software')
    self.assertEqual(mock_runSoftwareWithLock.call_count, 1)
    self.assertEqual(mock_runInstanceWithLock.call_count, 0)

    # Second, instanciation should start if compilation succeeded
    mock_runSoftwareWithLock.reset_mock()
    build_and_run_parameter_dict.update({'run_instance': True})
    runner_utils.saveBuildAndRunParams(config, build_and_run_parameter_dict)

    runner_utils.runSlapgridUntilSuccess(config, 'software')
    self.assertEqual(mock_runSoftwareWithLock.call_count, 1)
    self.assertEqual(mock_runInstanceWithLock.call_count, 1)

  @mock.patch('slapos.runner.utils.runInstanceWithLock')
  @mock.patch('slapos.runner.utils.runSoftwareWithLock')
  def test_runSoftwareDonotRestartForeverEvenIfBuildoutFileIsWrong(self,
                                                                   mock_runSoftwareWithLock,
                                                                   mock_runInstanceWithLock):
    """
    Restarting compilation or instanciation should happen a limited number of
    times to prevent useless runs due to a mistaken buildout config.
    """
    cwd = os.getcwd()
    config = {'runner_workdir': cwd,
              'etc_dir': cwd}

    build_and_run_parameter_dict = {
      'run_instance': True,
      'run_software': True,
      'max_run_instance': 3,
      'max_run_software': 3,
    }
    runner_utils.saveBuildAndRunParams(config, build_and_run_parameter_dict)

    # runSoftwareWithLock always fail and runInstanceWithLock succeeds on 1st try
    mock_runSoftwareWithLock.return_value = 1
    mock_runInstanceWithLock.return_value = 0

    runner_utils.runSlapgridUntilSuccess(config, 'software')
    self.assertEqual(mock_runSoftwareWithLock.call_count,
                     build_and_run_parameter_dict['max_run_software'])
    # if running software fails, then no need to try to deploy instances
    self.assertEqual(mock_runInstanceWithLock.call_count, 0)

  @mock.patch('os.path.exists')
  @mock.patch('slapos.runner.utils.updateInstanceParameter')
  @mock.patch('slapos.runner.utils.isSoftwareReleaseCompleted')
  @mock.patch('slapos.runner.utils.runSlapgridUntilSuccess')
  @mock.patch('slapos.runner.utils.isSoftwareRunning')
  @mock.patch('slapos.runner.utils.runSoftwareWithLock')
  def test_isSoftwareReleaseReady(self,
                                  mock_runSoftwareWithLock,
                                  mock_isSoftwareRunning,
                                  mock_runSlapgridUntilSuccess,
                                  mock_isSoftwareReleaseCompleted,
                                  mock_updateInstanceParameter,
                                  mock_path_exists):
    cwd = os.getcwd()
    config = {
      'etc_dir': cwd,
      'runner_workdir': cwd,
      'slapos-software': 'slapos/dummy',
      'auto_deploy': False,
      'autorun': False,
    }
    # Every parameter is False, so do nothing
    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '0')

    # We define a software, so from now we expect to build
    mock_path_exists.return_value = True

    # auto_deploy is True, so Software Release should build
    config.update({
      'auto_deploy': True,
    })

    # slapgrid-sr is running
    mock_isSoftwareRunning.return_value = True
    mock_isSoftwareReleaseCompleted.return_value = False
    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '2')
    
    # SR is not built, and slapgrid-sr is not running, so it should be started
    mock_isSoftwareRunning.return_value = False
    mock_isSoftwareReleaseCompleted.return_value = False

    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '2')
    self.assertTrue(mock_runSoftwareWithLock.called)

    mock_runSoftwareWithLock.reset_mock()

    # SR is built
    mock_isSoftwareReleaseCompleted.return_value = True
    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '1')

    # If autorun is True, Instance is expected to build too
    config.update({
      'autorun': True,
    })
    mock_isSoftwareRunning.return_value = False
    mock_isSoftwareReleaseCompleted.return_value = True

    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '1')
    mock_runSlapgridUntilSuccess.assert_called_with(config, 'instance')


  @unittest.skip('No scenario defined')
  def test_autoDeployWontEraseExistingInstances(self):
    raise NotImplementedError

  @unittest.skip('No scenario defined')
  def test_requestingInstanceCorrectlyPassesTypeAndParameters(self):
    raise NotImplementedError

  @unittest.skip('No scenario defined')
  def test_parametersAreCorrectlyUpdatedAndGivenToTheInstance(self):
    raise NotImplementedError


class TestViews(unittest.TestCase):
  def setUp(self):
    app = slapos.runner.views.app
    self.client = app.test_client()
    for key in (
        'etc_dir',
        'workspace',
        'software_link',
        'instance_profile',
        'instance_root',
    ):
      tmpdir = tempfile.mkdtemp()
      app.config[key] = tmpdir
      self.addCleanup(shutil.rmtree, tmpdir)

    slapos_command_dir = tempfile.mkdtemp()
    self.addCleanup(shutil.rmtree, slapos_command_dir)
    slapos_command = os.path.join(slapos_command_dir, 'slapos')
    self.slapos_command_arguments = os.path.join(slapos_command_dir, 'slapos.arguments')
    self.slapos_command_output = os.path.join(slapos_command_dir, 'slapos.output')
    with open(slapos_command, 'w') as f:
      f.write(textwrap.dedent('''\
          #!/bin/sh
          echo $@ > {self.slapos_command_arguments}
          cat {self.slapos_command_output}
      '''.format(**locals())))
    os.chmod(slapos_command, 0o700)
    app.config['slapos'] = slapos_command

    app.config['configuration_file_path'] = 'slapos.cfg'
    app.config['instance_monitoring_url'] = 'https://monitoring.example.com'
    app.config['master_url'] = 'https://slapos.example.com'
    app.config['computer_id'] = 'COMP-1234'
    app.config['partition_amount'] = 1
    app.config['software_profile'] = 'https://example.com/software.cfg'
    app.config['TESTING'] = True
    app.secret_key = 'secret'

  def test_home(self):
    self.assertIn(b'SlapOS web runner', self.client.get('/').data)

  def test_browseWorkspace(self):
    self.assertEqual(self.client.get('/browseWorkspace').status_code, 200)

  def test_editSoftwareProfile(self):
    self.assertEqual(self.client.get('/editSoftwareProfile').status_code, 200)

  def test_inspectSoftware(self):
    self.assertEqual(self.client.get('/inspectSoftware').status_code, 200)

  def test_editInstanceProfile(self):
    self.assertEqual(self.client.get('/editInstanceProfile').status_code, 200)

  @mock.patch('slapos.runner.utils.slapos.slap')
  def test_inspectInstance(self, _slap):
    with open(self.slapos_command_output, 'w') as f:
      f.write(textwrap.dedent('''
        slappart1:service            RUNNING   pid 1234, uptime 6 days, 4:02:18
        watchdog                     RUNNING   pid 5678, uptime 6 days, 4:03:07
        '''))

    self.assertEqual(
        json.loads(
            self.client.get(
                '/inspectInstance',
                headers={
                    'Accept': 'application/json'
                },
            ).data),
        [{
            'service_name':
            'slappart1:service',
            'status': 'RUNNING'
        }],
    )
    with open(self.slapos_command_arguments) as f:
      self.assertEqual(f.read().strip(), 'node supervisorctl --cfg slapos.cfg status')

    self.assertIn(
        b'<a href="/startStopProccess/name/slappart1:service/cmd/RUNNING">RUNNING</a>',
        self.client.get('/inspectInstance', headers={
            'Accept': '*/*'
        }).data)
    _slap.slap().initializeConnection.assert_called_with('https://slapos.example.com')
    _slap.slap().registerComputer.assert_called_with('COMP-1234')

  def test_supervisordStatus(self):
    with open(self.slapos_command_output, 'w') as f:
      f.write(
          textwrap.dedent('''
        slappart1:service            RUNNING   pid 1234, uptime 6 days, 4:02:18
        watchdog                     RUNNING   pid 5678, uptime 6 days, 4:03:07
        '''))

    self.assertEqual(
        json.loads(self.client.get('/supervisordStatus').data), {
            'code':
            1,
            'result':
            "<tr><th>Partition and Process name</th><th>Status</th>"
            "<th>Process PID </th><th> UpTime</th><th></th></tr><tr>"
            "<td  class='first'><b>"
            "<a href='/tailProcess/name/slappart1:service'>slappart1:service</a>"
            "</b></td><td align='center'>"
            "<a href='/startStopProccess/name/slappart1:service/cmd/RUNNING'>"
            "RUNNING</a></td><td align='center'>1234</td><td>6</td><td align='center'>"
            "<a href='/startStopProccess/name/slappart1:service/cmd/RESTART'>"
            "Restart</a></td></tr>"
        })

    def test_runInstanceProfile(self):
      self.assertEqual(self.client.get('/runInstanceProfile').status_code, 200)
      self.assertEqual(self.client.post('/runInstanceProfile').status_code, 200)

    def test_removeInstance(self):
      self.assertEqual(self.client.get('/removeInstance').status_code, 200)

    def test_viewLog(self):
      self.assertEqual(self.client.get('/viewLog').status_code, 200)

    def test_getFileLog(self):
      self.assertEqual(self.client.post('/getFileLog').status_code, 200)

    def test_stopAllPartition(self):
      self.assertEqual(self.client.post('/stopAllPartition').status_code, 302)

    def test_tailProcess(self):
      self.assertEqual(
          self.client.post('/tailProcess/name/slappart1:service').status_code,
          200)

    # TODO: test other important views


if __name__ == '__main__':
  random.seed()
  unittest.main()



# -*- coding: utf-8 -*-
# vim: set et sts=2:
# pylint: disable-msg=W0311,C0301,C0103,C0111

import codecs
import json
import os
import shutil
import subprocess
from . import sup_process
from six.moves import _thread
from six.moves.urllib.parse import unquote

from flask import (Flask, request, redirect, url_for, render_template,
                   g, flash, jsonify, session, abort, send_file)

import slapos
from slapos.util import bytes2str
from slapos.runner.utils import (checkSoftwareFolder, configNewSR, checkUserCredential,
                                 createNewUser, getBuildAndRunParams,
                                 getProfilePath, getSlapgridResult,
                                 html_escape,
                                 listFolder, getBuildAndRunParams,
                                 getProjectTitle, getRcode, updateUserCredential,
                                 getSlapStatus, getSvcStatus,
                                 getSvcTailProcess, getUsernameList, isInstanceRunning,
                                 isSoftwareRunning, isSoftwareReleaseReady, isText,
                                 loadSoftwareRList, md5sum, newSoftware,
                                 readFileFrom, readParameters, realpath,
                                 removeCurrentInstance,
                                 removeSoftwareByName, runSlapgridUntilSuccess,
                                 saveBuildAndRunParams,
                                 setMiniShellHistory,
                                 stopProxy,
                                 svcStartStopProcess, svcStartAll, svcStopAll, tail,
                                 updateInstanceParameter)

from git import GitCommandError
from slapos.runner.fileBrowser import FileBrowser
from slapos.runner.gittools import (cloneRepo, gitStatus, switchBranch,
                                    addBranch, getDiff, gitCommit, gitPush,
                                    gitPull, updateGitConfig, safeResult)


app = Flask(__name__)
app.config['MAX_CONTENT_LENGTH'] = 20 * 1024 * 1024
file_request = FileBrowser(app.config)

import logging
logger = logging.getLogger('slaprunner')

def login_redirect(*args, **kwargs):
  return redirect(url_for('login'))


@app.before_request
def before_request():
  if request.path.startswith('/static') \
    or request.path == '/isSRReady':
    return
  """
  account = getSession(app.config)
  if account:
    session['title'] = getProjectTitle(app.config)
  else:
    session['title'] = "No account is defined"
    if request.path != "/setAccount" and request.path != "/configAccount":
      return redirect(url_for('setAccount'))
  """
  session['title'] = getProjectTitle(app.config)
  g.instance_monitoring_url =  app.config['instance_monitoring_url']

# general views
def home():
  return render_template('index.html')


# general views
def browseWorkspace():
  return render_template('workspace.html')


@app.route("/login")
def login():
  return render_template('login.html')


@app.route("/setAccount")
def setAccount():
  """account = getSession(app.config)
  if not account:
    return render_template('account.html')
  return redirect(url_for('login'))"""


def myAccount():
  # account = getSession(app.config)
  git_user_path = os.path.join(app.config['etc_dir'], '.git_user')
  name = email = ""
  if os.path.exists(git_user_path):
    with open(git_user_path, 'r') as gfile:
      name, email = gfile.read().split(';')
  return render_template(
    'account.html',
    username_list=getUsernameList(app.config),
    email=email,
    name=name.decode('utf-8'),
    params=getBuildAndRunParams(app.config))


def getSlapgridParameters():
  return jsonify(getBuildAndRunParams(app.config))


def manageRepository():
  public_key = open(app.config['public_key']).read()
  account = [] #getSession(app.config)
  git_user_path = os.path.join(app.config['etc_dir'], '.git_user')
  name = email = ""
  if os.path.exists(git_user_path):
    with codecs.open(git_user_path, 'r', encoding='utf-8') as gfile:
      name, email = gfile.read().split(';')
  return render_template('manageRepository.html', workDir='workspace',
            project=listFolder(app.config, 'workspace'),
            public_key=public_key, name=name.decode('utf-8'),
            email=email)


@app.route("/doLogin", methods=['POST'])
def doLogin():
  #XXX Now has to check the .htpasswd if we want to warn
  #the user that he misspelled his name/password
  return jsonify(code=1, result="")


# software views
def editSoftwareProfile():
  profile = getProfilePath(app.config['etc_dir'], app.config['software_profile'])
  if profile == "":
    flash('Error: can not open profile, please select your project first')
  return render_template('updateSoftwareProfile.html', workDir='workspace',
      profile=profile, projectList=listFolder(app.config, 'workspace'))


def inspectSoftware():
  return render_template('runResult.html', softwareRoot='software_link/',
                         softwares=loadSoftwareRList(app.config))


#remove content of compiled software release
def removeSoftware():
  if isSoftwareRunning(app.config) or isInstanceRunning(app.config):
    flash('Software installation or instantiation in progress, cannot remove')
  elif os.path.exists(app.config['software_root']):
    svcStopAll(app.config)
    shutil.rmtree(app.config['software_root'])
    for link in os.listdir(app.config['software_link']):
      os.remove(os.path.join(app.config['software_link'], link))
    flash('Software removed')
  return redirect(url_for('inspectSoftware'))


def runSoftwareProfile():
  _thread.start_new_thread(runSlapgridUntilSuccess,
    (app.config, "software", True))
  return jsonify(result=True)


# instance views
def editInstanceProfile():
  profile = getProfilePath(app.config['etc_dir'], app.config['instance_profile'])
  if profile == "":
    flash('Error: can not open instance profile for this Software Release')
  return render_template('updateInstanceProfile.html', workDir='workspace',
      profile=profile, projectList=listFolder(app.config, 'workspace'))


# get status of all computer partitions and process state
def inspectInstance():
  if os.path.exists(app.config['instance_root']):
    file_path = 'instance_root'
    supervisor = getSvcStatus(app.config)
  else:
    file_path = ''
    supervisor = []
  if "application/json" in request.accept_mimetypes.best:
    result_list = []
    for service in supervisor:
      result_list.append({
        'service_name': service[0],
        'status': service[1],
      })
    return jsonify(result_list)
  return render_template('instanceInspect.html',
                         file_path=file_path,
                         supervisor=supervisor,
                         slap_status=getSlapStatus(app.config),
                         partition_amount=app.config['partition_amount'])

def getConnectionParameter(partition_reference):
  """
  Return connection parameters of a partition.
  """
  slap = slapos.slap.slap()
  slap.initializeConnection(app.config['master_url'])
  partition = slap.registerComputerPartition(app.config['computer_id'], partition_reference)
  return jsonify(partition.getConnectionParameterDict())
app.add_url_rule("/getConnectionParameter/<partition_reference>", 'getConnectionParameter', getConnectionParameter, methods=['GET'])

#Reload instance process ans returns new value to ajax
def supervisordStatus():
  result = getSvcStatus(app.config)
  if not result:
    return jsonify(code=0, result="")
  # XXX-Marco -> template
  html = "<tr><th>Partition and Process name</th><th>Status</th><th>Process PID </th><th> UpTime</th><th></th></tr>"
  for item in result:
    html += "<tr>"
    html += "<td  class='first'><b><a href='" + url_for('tailProcess', process=item[0]) + "'>" + item[0] + "</a></b></td>"
    html += "<td align='center'><a href='" + url_for('startStopProccess', process=item[0], action=item[1]) + "'>" + item[1] + "</a></td>"
    html += "<td align='center'>" + item[3] + "</td><td>" + item[5] + "</td>"
    html += "<td align='center'><a href='" + url_for('startStopProccess', process=item[0], action='RESTART') + "'>Restart</a></td>"
    html += "</tr>"
  return jsonify(code=1, result=html)


def removeInstance():
  logger.warning("User clicked on 'Destroy All Services'. Removing all instances...")
  result = removeCurrentInstance(app.config)
  if isinstance(result, str):
    flash(result)
  return redirect(url_for('inspectInstance'))


def runInstanceProfile():
  if not os.path.exists(app.config['instance_root']):
    os.mkdir(app.config['instance_root'])
  _thread.start_new_thread(runSlapgridUntilSuccess,
    (app.config, "instance", True))
  return jsonify(result=True)


def viewLog():
  return render_template('viewLog.html')

def getFileLog():
  logfile = request.form.get('filename', '').encode('utf-8')
  if logfile == "instance.log":
    file_path = app.config['instance_log']
  elif logfile == "software.log":
    file_path = app.config['software_log']
  else:
    file_path = realpath(app.config, logfile)
  try:
    if not os.path.exists(file_path):
      raise IOError
    if not isText(file_path):
      content = "Can not open binary file, please select a text file!"
    with open(file_path) as f:
      if 'truncate' in request.form:
        content = tail(f, int(request.form['truncate']))
      else:
        content = f.read()
    return jsonify(code=1, result=html_escape(content))
  except Exception:
    return jsonify(code=0, result="Warning: Log file doesn't exist yet or empty log!!")


def stopAllPartition():
  svcStopAll(app.config)
  return redirect(url_for('inspectInstance'))

def startAllPartition():
  svcStartAll(app.config)
  return redirect(url_for('inspectInstance'))

def tailProcess(process):
  return render_template('processTail.html',
      process_log=getSvcTailProcess(app.config, process), process=process)


def startStopProccess(process, action):
  svcStartStopProcess(app.config, process, action)
  return redirect(url_for('inspectInstance'))


def openProject(method):
  return render_template('projectFolder.html', method=method,
                         workDir='workspace')


def cloneRepository():
  path = realpath(app.config, request.form['name'], False)
  if not path:
    return jsonify(code=0, result="Can not access folder: Permission Denied")
  try:
    cloneRepo(request.form['repo'], path, request.form['user'], request.form['email'])
    return jsonify(code=1, result="")
  except GitCommandError as e:
    return jsonify(code=0, result=safeResult(str(e)))


def listDirectory():
  folderList = listFolder(app.config, request.form['name'])
  return jsonify(result=folderList)


def createSoftware():
  return newSoftware(request.form['folder'], app.config, session)


def checkFolder():
  return checkSoftwareFolder(request.form['path'], app.config)


def setCurrentProject():
  if configNewSR(app.config, request.form['path']):
    session['title'] = getProjectTitle(app.config)
    return jsonify(code=1, result="")
  else:
    return jsonify(code=0, result=("Can not setup this Software Release"))


def getProjectStatus():
  path = realpath(app.config, request.form['project'])
  if path:
    try:
      result, branch, isdirty = gitStatus(path)
      return jsonify(code=1, result=result, branch=branch, dirty=isdirty)
    except GitCommandError as e:
      return jsonify(code=0, result=safeResult(str(e)))
  else:
    return jsonify(code=0, result="Can not read folder: Permission Denied")


#view for current software release files
def editCurrentProject():
  project_file = os.path.join(app.config['etc_dir'], ".project")
  # XXX hardcoded default project
  project = "workspace/slapos"
  if os.path.exists(project_file):
    project = open(project_file).read()
  projectList = listFolder(app.config, 'workspace')
  if projectList:
    return render_template('softwareFolder.html', workDir='runner_workdir',
                           project=project,
                           projectList=projectList)
  else:
    flash('Please clone slapos repository, or your own repository')
    return redirect(url_for('manageRepository'))

#create file or directory
def createFile():
  path = realpath(app.config, request.form['file'], False)
  if not path:
    return jsonify(code=0, result="Error when creating your %s: Permission Denied" % request.form['type'])
  try:
    if request.form['type'] == "file":
      open(path, 'w')
    else:
      os.mkdir(path)
    return jsonify(code=1, result="")
  except Exception as e:
    return jsonify(code=0, result=str(e))


#remove file or directory
def removeFile():
  try:
    if request.form['type'] == "folder":
      shutil.rmtree(request.form['path'])
    else:
      os.remove(request.form['path'])
    return jsonify(code=1, result="")
  except Exception as e:
    return jsonify(code=0, result=str(e))


def removeSoftwareDir():
    status, message = removeSoftwareByName(app.config, request.form['md5'],
      request.form['title'])
    return jsonify(code=status, result=message)


#read file and return content to ajax
def getFileContent():
  file_path = realpath(app.config, request.form['file'])
  if file_path:
    if not isText(file_path):
      return jsonify(code=0,
            result="Can not open a binary file, please select a text file!")
    if 'truncate' in request.form:
      content = tail(codecs.open(file_path, "r", "utf_8"), int(request.form['truncate']))
      return jsonify(code=1, result=content)
    else:
      return jsonify(code=1, result=codecs.open(file_path, "r", "utf_8").read())
  else:
    return jsonify(code=0, result="Error: No such file!")


def saveFileContent():
  file_path = realpath(app.config, request.form['file'], False)
  if file_path:
    with open(file_path, 'w') as f:
      f.write(request.form['content'].encode("utf-8"))
    return jsonify(code=1, result="")
  else:
    return jsonify(code=0, result="Rejected!! Cannot access to this location.")


def changeBranch():
  path = realpath(app.config, request.form['project'])
  if path:
    try:
      if switchBranch(path, request.form['name']):
        return jsonify(code=1, result="")
      else:
        json = "This is already your active branch for this project"
        return jsonify(code=1, result=json)
    except GitCommandError as e:
      return jsonify(code=0, result=safeResult(str(e)))
  else:
    return jsonify(code=0, result="Can not read folder: Permission Denied")


def newBranch():
  path = realpath(app.config, request.form['project'])
  if path:
    try:
      if request.form['create'] == '1':
        result = addBranch(path, request.form['name'])
      else:
        result =  addBranch(path, request.form['name'], True)
      if result:
        return jsonify(code=1, result="")
      else:
        return jsonify(code=0, result="Failed to checkout to branch %s.")
    except GitCommandError as e:
      return jsonify(code=0, result=safeResult(str(e)))
  else:
    return jsonify(code=0, result="Can not read folder: Permission Denied")


def getProjectDiff():
  path = realpath(app.config, request.form['project'])
  if path:
    return jsonify(code=1, result=getDiff(path))
  else:
    return jsonify(code=0,
                  result="Error: No such file or directory. PERMISSION DENIED!")


def commitProjectFiles():
  path = realpath(app.config, request.form['project'])
  if path:
    return gitCommit(path, request.form['msg'])
  else:
    return jsonify(code=0, result="Can not read folder: Permission Denied")


def pushProjectFiles():
  path = realpath(app.config, request.form['project'])
  if path:
    return gitPush(path)
  else:
    return jsonify(code=0, result="Can not read folder: Permission Denied")


def pullProjectFiles():
  path = realpath(app.config, request.form['project'])
  if path:
    return gitPull(path)
  else:
    return jsonify(code=0, result="Can not read folder: Permission Denied")


def checkFileType():
  path = realpath(app.config, request.form['path'])
  if not path:
    return jsonify(code=0, result="Can not open file: Permission Denied!")
  if isText(path):
    return jsonify(code=1, result="text")
  else:
    return jsonify(code=0,
                   result="Can not open a binary file, please select a text file!")


def getmd5sum():
  realfile = realpath(app.config, request.form['file'])
  if not realfile:
    return jsonify(code=0, result="Can not open file: Permission Denied!")
  md5 = md5sum(realfile)
  if md5:
    return jsonify(code=1, result=md5)
  else:
    return jsonify(code=0, result="Can not get md5sum for this file!")


#return information about state of slapgrid process
def slapgridResult():
  software_state = isSoftwareRunning(app.config)
  instance_state = isInstanceRunning(app.config)
  log_result = {
    'content': '',
    'position': 0,
    'truncated': False
  }
  if request.form['log'] in ['software', 'instance']:
    log_file = request.form['log'] + "_log"
    if os.path.exists(app.config[log_file]):
      with open(app.config[log_file], 'rb') as f:
        log_result = bytes2str(readFileFrom(f, int(request.form['position'])))
  build_result = getSlapgridResult(app.config, 'software')
  run_result = getSlapgridResult(app.config, 'instance')
  software_info = {'state':software_state,
                   'last_build':build_result['last_build'],
                   'success':build_result['success']}
  instance_info = {'state':instance_state,
                   'last_build':run_result['last_build'],
                   'success':run_result['success']}
  return jsonify(software=software_info, instance=instance_info,
                 result=(instance_state or software_state), content=log_result)


def stopSlapgrid():
  counter_file = os.path.join(app.config['runner_workdir'], '.turn-left')
  open(counter_file, 'w+').write(str(0))
  result = sup_process.killRunningProcess(app.config, request.form['type'])
  return jsonify(result=result)


def getPath():
  files = request.form['file'].split('#')
  list = []
  for p in files:
    path = realpath(app.config, p)
    if not path:
      list = []
      break
    else:
      list.append(path)
  realfile = '#'.join(list)
  if not realfile:
    return jsonify(code=0,
                   result="Can not access to this file: Permission Denied!")
  else:
    return jsonify(code=1, result=realfile)


def saveParameterXml():
  """
  Update instance parameter into a local xml file.
  """
  project = os.path.join(app.config['etc_dir'], ".project")
  if not os.path.exists(project):
    return jsonify(code=0, result="Please first open a Software Release")
  content = request.form['parameter'].encode("utf-8")
  param_path = os.path.join(app.config['etc_dir'], ".parameter.xml")
  try:
    with open(param_path, 'w') as f:
      f.write(content)
    result = readParameters(param_path)
  except Exception as e:
    result = str(e)
  software_type = None
  if request.form['software_type']:
    software_type = request.form['software_type']
  if type(result) == type(''):
    return jsonify(code=0, result=result)
  else:
    try:
      updateInstanceParameter(app.config, software_type)
    except Exception as e:
      return jsonify(
        code=0,
        result="An error occurred while applying your settings!<br/>%s" % e)
    return jsonify(code=1, result="")


def getSoftwareType():
  software_type_path = os.path.join(app.config['etc_dir'], ".software_type.xml")
  if os.path.exists(software_type_path):
    return jsonify(code=1, result=open(software_type_path).read())
  return jsonify(code=1, result="default")


#read instance parameters into the local xml file and return a dict
def getParameterXml(request):
  param_path = os.path.join(app.config['etc_dir'], ".parameter.xml")
  if not os.path.exists(param_path):
    default = '<?xml version="1.0" encoding="utf-8"?>\n<instance>\n</instance>'
    return jsonify(code=1, result=default)
  if request == "xml":
    parameters = open(param_path).read()
  else:
    parameters = readParameters(param_path)
  if type(parameters) == type('') and request != "xml":
    return jsonify(code=0, result=parameters)
  else:
    return jsonify(code=1, result=parameters)


#update user-defined slapgrid parameters
def updateBuildAndRunConfig():
  code = 1
  try:
    max_run_instance = int(request.form['max_run_instance'].strip())
    max_run_software = int(request.form['max_run_software'].strip())
  except ValueError:
    code = 0
    result = "Error! You should have provided an integer"
  run_instance = (True if request.form['run_instance']=="true" else False)
  run_software = (True if request.form['run_software']=="true" else False)
  if code:
    params =  {}
    params['run_instance'] = run_instance
    params['run_software'] = run_software
    params['max_run_instance'] = max_run_instance
    params['max_run_software'] = max_run_software
    saveBuildAndRunParams(app.config, params)
    result = "Your parameters have correctly been updated"
  return jsonify(code=code, result=result)


#update user account data
def updateAccount():

  username = request.form['username'].strip()
  password = request.form['password'].strip()
  new_password = request.form['new_password'].strip()
  email = request.form['email'].strip()
  name = request.form['name'].strip()

  if username and password and new_password:
    if not checkUserCredential(app.config, username, password):
      return jsonify(code=0, result="Username and password doesn't match for '%s'" % username)

    result = updateUserCredential(app.config, username, new_password)
    if not result:
      return jsonify(code=0, result="Failed to update Credentials!")

  try:
    updateGitConfig(app.config['default_repository_path'], name, email)
  except GitCommandError as e:
    return jsonify(code=0, result=str(e))
  git_user_file = os.path.join(app.config['etc_dir'], '.git_user')
  with codecs.open(git_user_file, 'w', encoding='utf-8') as gfile:
    gfile.write(u'{};{}'.format(name, email))
  return jsonify(code=1, result="")


#update user account data
@app.route("/configAccount", methods=['POST'])
def configAccount():
  """last_account = getSession(app.config)
  if last_account:
    return jsonify(code=0,
                   result="Unable to respond to your request, permission denied.")

  account = []
  account.append(request.form['username'].strip())
  account.append(request.form['password'].strip())
  account.append(request.form['email'].strip())
  account.append(request.form['name'].strip())
  code = request.form['rcode'].strip()
  recovery_code = getRcode(app.config)
  if code != recovery_code:
    return jsonify(code=0, result="Your password recovery code is not valid!")
  result = saveSession(app.config, account)
  if type(result) == type(""):
    return jsonify(code=0, result=result)
  else:
    return jsonify(code=1, result="")"""

def addUser():
  if createNewUser(app.config, request.form['username'],
                   request.form['password']):
    return jsonify(code=1, result="New user succesfully saved")
  else:
    return jsonify(code=0, result="Problem while creating new user")


#Global File Manager
def fileBrowser():
  if request.method == 'POST':
    filename = request.form.get('filename', '').encode('utf-8')
    dir = request.form['dir'].encode('utf-8')
    newfilename = request.form.get('newfilename', '').encode('utf-8')
    files = request.form.get('files', '').encode('utf-8')
    opt = request.form.get('opt')
    opt = int(opt) if opt else 1
  else:
    opt = int(request.args.get('opt'))

  try:
    if opt == 1:
      #list files and directories
      result = file_request.listDirs(dir)
    elif opt == 2:
      #Create file
      result = file_request.makeFile(dir, filename)
    elif opt == 3:
      #Create directory
      result = file_request.makeDirectory(dir, filename)
    elif opt == 4:
      #Delete a list of files or/and directories
      result = file_request.deleteItem(dir, files)
    elif opt == 5:
      #copy a lis of files or/and directories
      result = file_request.copyItem(dir, files)
    elif opt == 6:
      #rename file or directory
      result = file_request.rename(dir, filename, newfilename)
    elif opt == 7:
      result = file_request.copyItem(dir, files, del_source=True)
    elif opt == 8:
      #donwload file
      filename = request.args.get('filename').encode('utf-8')
      result = file_request.downloadFile(request.args.get('dir').encode('utf-8'),
                filename)
      try:
        return send_file(result, attachment_filename=filename, as_attachment=True)
      except Exception:
        abort(404)
    elif opt == 9:
      result = file_request.readFile(dir, filename, False)
    elif opt == 11:
      #Upload file
      result = file_request.uploadFile(dir, request.files)
    elif opt == 14:
      #Copy file or directory as ...
      result = file_request.copyAsFile(dir, filename, newfilename)
    elif opt == 16:
      #zip file
      result = file_request.zipFile(dir, filename, newfilename)
    elif opt == 17:
      #zip file
      result = file_request.unzipFile(dir, filename, newfilename)
    elif opt == 20:
      #Fancy Load folder
      key = request.args.get('key')
      dir = request.args.get('dir').encode('utf-8')
      listfiles = request.args.get('listfiles', '')
      data = file_request.fancylistDirs(dir, key, listfiles)
      result = json.dumps(data)
    else:
      result = "ARGS PARSE ERROR: Bad option..."
  except Exception as e:
    return str(e)
  return result


def editFile():
  return render_template('editFile.html', workDir='workspace',
    profile=unquote(request.args.get('profile', '')),
    projectList=listFolder(app.config, 'workspace'),
    filename=unquote(request.args.get('filename', '')))

def shell():
  return render_template('shell.html')


def isSRReady():
  return isSoftwareReleaseReady(app.config)


def runCommand():
  cwd = open(app.config['minishell_cwd_file'], 'r').read().strip()
  command = request.form.get("command", '').strip()
  parsed_commands = command.split(';');
  # does the user want to change current directory ?
  for cmd in parsed_commands:
    if 'cd' == cmd[:2]:
      cmd = cmd.split(' ');
      real_cmd = cmd[:]
      if len(cmd) == 1:
        cmd.append(os.environ.get('HOME'))
      # shorten directory's name, to avoid things like : /a/../b
      cd_dir = os.path.realpath(os.path.join(cwd, cmd[1]))
      if os.path.exists(cd_dir) and os.path.isdir(cd_dir):
        cwd = cd_dir
        # save new cwd in the config file
        open(app.config['minishell_cwd_file'], 'w').write(cwd)
        # if the command was just cd, execute it. Otherwise, execute the rest
        command = command.replace(' '.join(real_cmd), '').strip(';')
        if not command:
          return jsonify(path=cwd, data="Changed directory, now in : "+cwd)
  try:
    setMiniShellHistory(app.config, command)
    command = "timeout 600 " + command
    return jsonify(path=cwd, data=subprocess.check_output(command, stderr=subprocess.STDOUT, shell=True, cwd=cwd, env={'PATH': app.config['path']}))
  except subprocess.CalledProcessError as e:
    return jsonify(path=cwd, data=e.output, returncode=e.returncode, error=True)


def getMiniShellHistory():
  history_file = app.config['minishell_history_file']
  if not os.path.exists(history_file):
    return ""
  history = open(history_file, 'r').readlines()
  for line, text in enumerate(history):
    history[line] = text.strip()
  return json.dumps(history)


#Setup List of URLs
app.add_url_rule('/', 'home', home)
app.add_url_rule('/browseWorkspace', 'browseWorkspace', browseWorkspace)
app.add_url_rule('/editSoftwareProfile', 'editSoftwareProfile',
                editSoftwareProfile)
app.add_url_rule('/inspectSoftware', 'inspectSoftware', inspectSoftware)
app.add_url_rule('/removeSoftware', 'removeSoftware', removeSoftware)
app.add_url_rule('/runSoftwareProfile', 'runSoftwareProfile',
                 runSoftwareProfile, methods=['GET', 'POST'])
app.add_url_rule('/editInstanceProfile', 'editInstanceProfile',
                 editInstanceProfile)
app.add_url_rule('/inspectInstance', 'inspectInstance',
                 inspectInstance, methods=['GET'])
app.add_url_rule('/supervisordStatus', 'supervisordStatus',
                 supervisordStatus, methods=['GET'])
app.add_url_rule('/runInstanceProfile', 'runInstanceProfile',
                 runInstanceProfile, methods=['GET', 'POST'])
app.add_url_rule('/removeInstance', 'removeInstance', removeInstance)
app.add_url_rule('/viewLog', 'viewLog',
                 viewLog, methods=['GET'])
app.add_url_rule("/getFileLog", 'getFileLog', getFileLog,
                 methods=['POST'])
app.add_url_rule('/stopAllPartition', 'stopAllPartition',
                 stopAllPartition, methods=['GET'])
app.add_url_rule('/startAllPartition', 'startAllPartition',
                 startAllPartition, methods=['GET'])
app.add_url_rule('/tailProcess/name/<process>', 'tailProcess',
                 tailProcess, methods=['GET'])
app.add_url_rule('/startStopProccess/name/<process>/cmd/<action>',
                 'startStopProccess', startStopProccess, methods=['GET'])
app.add_url_rule("/getParameterXml/<request>", 'getParameterXml',
                 getParameterXml, methods=['GET'])
app.add_url_rule('/getSoftwareType', 'getSoftwareType',
                 getSoftwareType, methods=['GET'])
app.add_url_rule("/stopSlapgrid", 'stopSlapgrid', stopSlapgrid, methods=['POST'])
app.add_url_rule("/slapgridResult", 'slapgridResult',
                 slapgridResult, methods=['POST'])
app.add_url_rule("/getmd5sum", 'getmd5sum', getmd5sum, methods=['POST'])
app.add_url_rule("/checkFileType", 'checkFileType', checkFileType,
                 methods=['POST'])
app.add_url_rule("/commitProjectFiles", 'commitProjectFiles', commitProjectFiles,
                 methods=['POST'])
app.add_url_rule("/pullProjectFiles", 'pullProjectFiles', pullProjectFiles,
                 methods=['POST'])
app.add_url_rule("/pushProjectFiles", 'pushProjectFiles', pushProjectFiles,
                 methods=['POST'])
app.add_url_rule("/getProjectDiff", 'getProjectDiff', getProjectDiff,
                 methods=['POST'])
app.add_url_rule("/newBranch", 'newBranch', newBranch, methods=['POST'])
app.add_url_rule("/changeBranch", 'changeBranch', changeBranch, methods=['POST'])
app.add_url_rule("/saveFileContent", 'saveFileContent', saveFileContent,
                 methods=['POST'])
app.add_url_rule("/removeSoftwareDir", 'removeSoftwareDir', removeSoftwareDir,
                 methods=['POST'])
app.add_url_rule("/getFileContent", 'getFileContent', getFileContent,
                 methods=['POST'])
app.add_url_rule("/removeFile", 'removeFile', removeFile, methods=['POST'])
app.add_url_rule("/createFile", 'createFile', createFile, methods=['POST'])
app.add_url_rule("/editCurrentProject", 'editCurrentProject', editCurrentProject)
app.add_url_rule("/getProjectStatus", 'getProjectStatus', getProjectStatus,
                 methods=['POST'])
app.add_url_rule('/openProject/<method>', 'openProject', openProject,
                 methods=['GET'])
app.add_url_rule("/setCurrentProject", 'setCurrentProject', setCurrentProject,
                 methods=['POST'])
app.add_url_rule("/checkFolder", 'checkFolder', checkFolder, methods=['POST'])
app.add_url_rule('/createSoftware', 'createSoftware', createSoftware,
                 methods=['POST'])
app.add_url_rule('/cloneRepository', 'cloneRepository', cloneRepository,
                 methods=['POST'])
app.add_url_rule('/listDirectory', 'listDirectory', listDirectory, methods=['POST'])
app.add_url_rule('/manageRepository', 'manageRepository', manageRepository)
app.add_url_rule("/saveParameterXml", 'saveParameterXml', saveParameterXml,
                 methods=['POST'])
app.add_url_rule("/getPath", 'getPath', getPath, methods=['POST'])
app.add_url_rule("/myAccount", 'myAccount', myAccount)
app.add_url_rule("/updateAccount", 'updateAccount', updateAccount,
                 methods=['POST'])
app.add_url_rule("/updateBuildAndRunConfig", 'updateBuildAndRunConfig', updateBuildAndRunConfig,
                 methods=['POST'])
app.add_url_rule("/fileBrowser", 'fileBrowser', fileBrowser,
                 methods=['GET', 'POST'])
app.add_url_rule("/editFile", 'editFile', editFile, methods=['GET'])
app.add_url_rule('/shell', 'shell', shell)
app.add_url_rule('/isSRReady', 'isSRReady', isSRReady)
app.add_url_rule('/addUser', 'addUser', addUser, methods=['POST'])
app.add_url_rule('/getSlapgridParameters', 'getSlapgridParameters', getSlapgridParameters, methods=['GET'])
app.add_url_rule('/runCommand', 'runCommand', runCommand, methods=['POST'])
app.add_url_rule("/getMiniShellHistory", 'getMiniShellHistory', getMiniShellHistory, methods=['GET'])



##############################################################################
#
# Copyright (c) 2019 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.grid.promise import PromiseError
from slapos.test.promise.plugin import TestPromisePluginMixin

import tempfile
import os
import six
import shutil


class TestCheckCommandExecute(TestPromisePluginMixin):
  def _createScript(self, filename, content):
    with open(filename, 'w') as fh:
      fh.write(content)
    os.chmod(filename, 0o755)

  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    self.tempdir = tempfile.mkdtemp()
    self.fail_command = os.path.join(self.tempdir, 'fail')
    self._createScript(self.fail_command, """#/bin/sh
echo failure
echo me 1>&2
exit 1""")
    self.success_command = os.path.join(self.tempdir, 'success')
    self._createScript(self.success_command, """#/bin/sh
echo success
echo me 1>&2
exit 0""")
    self.promise_name = "check-command-execute.py"

    self.base_content = """from slapos.promise.plugin.check_command_execute import RunPromise

extra_config_dict = {
  'command': %(command)r
}
"""

  def tearDown(self):
    TestPromisePluginMixin.tearDown(self)
    shutil.rmtree(self.tempdir)

  def test_check_success(self):
    self.writePromise(self.promise_name, self.base_content % {
      'command': self.success_command})
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      "OK %r run with success" % (self.success_command,)
    )

  def test_check_failure(self):
    self.writePromise(self.promise_name, self.base_content % {
      'command': self.fail_command})
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR %r run with failure, output: %s'failure\\nme\\n'" % (
         self.fail_command, 'b' if six.PY3 else '')
    )



#!/usr/bin/env python

"""
Check if a apachedex result matches the desired threshold or raises an error.
"""

import json
import os
import re
import sys
import time
import datetime
import argparse

def checkApachedexResult(apachedex_path, apachedex_report_status_file, desired_threshold):
  message = "No apachedex result for today or yesterday"
  today = datetime.date.today()
  today_or_yesterday = today, today - datetime.timedelta(1)
  try:
    apachedex_file = max(os.listdir(apachedex_path),
      key=lambda x: os.stat(os.path.join(apachedex_path, x)).st_mtime)
  except ValueError:
    if datetime.date.fromtimestamp(os.stat(apachedex_path).st_mtime) in today_or_yesterday: 
      return 0, "Instance has been just deployed. Skipping check.."
  else:
    for date in today_or_yesterday:
      if apachedex_file == date.strftime('ApacheDex-%Y-%m-%d.html'):

        with open(os.path.join(apachedex_path, apachedex_file)) as f:
          content = f.read()
        if content:
          # XXX: if not a lot of usage, skip
          # XXX: too fragile, use lxml.html and use xpath
          regex = r"Overall<\/h2>.*\n<th>apdex<\/th><th>.*?\n<\/tr><tr>\n<td [^<]*>(.*?)%<\/td>"
          m = re.findall(regex, content)
          if m:
            apx_result=int(m[0])
            if apx_result > desired_threshold:
              return 0, "OK - Score: {}%".format(apx_result)
            else:
              return 1, "Score too low: {}% - Threshold: {}%".format(apx_result, desired_threshold)
        message = "No result found in the apdex file or the file is corrupted"
        break

  if not os.path.exists(apachedex_report_status_file):
    return 0, "ApacheDex didn't run yet!"
  with open(apachedex_report_status_file) as f:
    try:
      json_content = json.load(f)
    except ValueError as e:
      json_content = ''
  if json_content:
    message += "\n" + json_content["message"]
  return 1, message

def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("--apachedex_path", required=True)
  parser.add_argument("--status_file", required=True)
  parser.add_argument("--threshold", required=True, type=float)
  args = parser.parse_args()

  status, message = checkApachedexResult(args.apachedex_path, args.status_file, args.threshold)
  print(message)
  sys.exit(status)



#!/usr/bin/env python

"""
Check if a mariadb result matches the desired threshold or raises an error.
"""

from __future__ import print_function

import json
import os
import re
import sys
import time
import datetime
import argparse
from backports import lzma

def checkMariadbDigestResult(mariadbdex_path, mariadbdex_report_status_file,
                             max_query_threshold, slowest_query_threshold):
  message = "No mariadbdex result for today or yesterday"
  today = datetime.date.today()
  today_or_yesterday = today, today - datetime.timedelta(1)

  try:
    mariadbdex_file = max(os.listdir(mariadbdex_path),
      key=lambda x: os.stat(os.path.join(mariadbdex_path, x)).st_mtime)
  except ValueError:
    if datetime.date.fromtimestamp(os.stat(mariadbdex_path).st_mtime) in today_or_yesterday: 
        return 0, "Instance has been just deployed. Skipping check.."
  else:
    for date in today_or_yesterday:
      if mariadbdex_file == date.strftime('slowquery_digest.txt-%Y-%m-%d.xz'):
        with lzma.open(os.path.join(mariadbdex_path, mariadbdex_file), 'rt') as f:
          content = f.read()
        if content:
          # XXX: if not a lot of usage, skip this
          regex = r"Overall: (.*) total,[\S\s]*# Exec time( *([\d]+)m?s?){4}"
          m = re.findall(regex, content)
          if m:
            total_queries_exec=m[0][0].strip()
            slowest_query_time=int(m[0][2].strip())
            has_k=total_queries_exec[-1:]
            if has_k == "k":
              pre=total_queries_exec[:-1]
              total_queries_exec=float(pre)*1000
            else:
              total_queries_exec=int(total_queries_exec)
            if total_queries_exec < max_query_threshold and slowest_query_time < slowest_query_threshold:
              return 0,  "Thanks for keeping it all clean, total queries are : " + str(total_queries_exec) + \
                         " slowest query time is : " + str(slowest_query_time)
            else:
              return 1, "Threshold is lower than expected: \nExpected total queries : " + \
                       str(max_query_threshold) +" and current is: " + str(total_queries_exec) + "\n"+ \
                       "Expected slowest query : " + str(slowest_query_threshold) + " and current is: " + \
                       str(slowest_query_time)
        message = "No result found in the slow query digest file or the file is corrupted"
        break

  if not os.path.exists(mariadbdex_report_status_file):
    return 0, "No slow query result yet!"
  with open(mariadbdex_report_status_file) as f:
    try:
      json_content = json.load(f)
    except ValueError as e:
      json_content = ''
  if json_content:
    message += "\n" + json_content["message"]
  return 1, message

def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("--ptdigest_path", required=True)
  parser.add_argument("--status_file", required=True)
  parser.add_argument("--max_queries_threshold", required=True, type=float)
  parser.add_argument("--slowest_query_threshold", required=True, type=float)
  args = parser.parse_args()

  status, message = checkMariadbDigestResult(args.ptdigest_path, args.status_file,
                                    args.max_queries_threshold, args.slowest_query_threshold
                                    )

  print(message)
  sys.exit(status)



# -*- coding: utf-8 -*-

import os
import subprocess
import logging
import signal

class SlapContainerError(Exception):
    """This exception is thrown, if there is
    any failure during slapcontainer preparation,
    starting or stopping process"""



def main(sr_directory, partition_list, database, bridge_name):

    logger = logging.getLogger('process')

    ######################
    # Process partitions #
    ######################
    start_requested = set()
    logger.debug('Processing partitions...')
    for partition_path in partition_list:
        partition_logger = logger.getChild(
            os.path.basename(partition_path)
        )
        partition_logger.debug('Processing...')

        # XXX: Hardcoded path
        slapcontainer_filename = os.path.join(partition_path,
                                              '.slapcontainername')
        if os.path.isfile(slapcontainer_filename):
            partition_logger.debug('Container found...')
            with open(slapcontainer_filename, 'r') as slapcontainer_file:
                name = slapcontainer_file.read().strip()

            # XXX: Hardcoded path
            lxc_conf_path = os.path.join(partition_path,
                                         'etc/lxc.conf')
            lxc_state_path = os.path.join(partition_path,
                                          '.slapcontainer.state')
            sib_state_path = os.path.join(partition_path,
                                          '.shellinabox.state')
            # XXX: Avoid hacking slapos.core
            ##########################################################
            magic_string = '!!BRIDGE_NAME!!'
            with open(lxc_conf_path, 'r') as lxc_conf_file:
                lxc_conf_content = lxc_conf_file.read()
            if magic_string in lxc_conf_content:
                with open(lxc_conf_path, 'w') as lxc_conf_file:
                    lxc_conf_file.write(
                        lxc_conf_content.replace(magic_string, bridge_name)
                    )
            ##########################################################
            with open(lxc_conf_path, 'r') as lxc_conf_file:
                requested_status = lxc_conf_file.readline().strip(' \n\r\t#')

            if requested_status == 'started':
                start_requested.add(name)

            process_partition(requested_status=requested_status,
                              sr_directory=sr_directory,
                              partition_path=partition_path,
                              name=name,
                              database=database,
                              logger=partition_logger,
                              lxc_conf_filename=lxc_conf_path,
                              lxc_state_path=lxc_state_path,
                              sib_state_path=sib_state_path)

    if start_requested:
        logger.debug('Container which start was requested : %s.',
                     ', '.join(start_requested))


    ####################################
    # Stop unwanted running containers #
    ####################################
    try:
        active_containers = set((container.strip() for container in call(
            [os.path.join(sr_directory, 'parts/lxc/bin/lxc-ls'),
             '--active']
        ).split('\n') if container.strip()))
        logger.debug('Active containers are %s.', ', '.join(active_containers))
    except SlapContainerError:
        active_containers = set()

    ### Stop containers
    to_stop = active_containers - start_requested
    if to_stop:
        logger.debug('Stopping containers %s.', ', '.join(to_stop))
    else:
        logger.debug('No extra containers to stop.')

    for container in to_stop:
        try:
            logger.info('Stopping container %s.', container)
            call(
                [os.path.join(sr_directory, 'parts/lxc/bin/lxc-stop'),
                 '-n', container
                ]
            )
        except SlapContainerError:
            logger.fatal('Impossible to stop %s.', container)

    ### Stop shellinaboxes
    active_shellinabox = set(database.keys())
    to_stop = active_shellinabox - start_requested
    if to_stop:
        logger.debug('Stopping shellinaboxes %s.', ', '.join(to_stop))
    else:
        logger.debug('No extra shellinabox to stop.')

    for shellinabox in to_stop:
        pid = int(database[shellinabox])
        try:
            os.kill(pid, signal.SIGTERM)
        except OSError:
            # Shellinabox's already stopped
            del database[shellinabox]
        else:
            # Stopping shellinabox
            if not is_pid_running(pid):
                del database[shellinabox]



def process_partition(requested_status,
                      sr_directory,
                      partition_path,
                      name,
                      database,
                      logger,
                      lxc_conf_filename,
                      lxc_state_path,
                      sib_state_path):

    if requested_status == 'started':

        ##############################
        # Stateless container launch #
        ##############################

        logger.debug('Check status.')
        lxc_info = call([os.path.join(sr_directory, 'parts/lxc/bin/lxc-info'),
                         '-n', name])
        ### Check if container is launched
        if 'RUNNING' in lxc_info:
            current_status = 'started'
        else:
            current_status = 'stopped'

        with open(lxc_state_path, 'w') as lxc_state_file:
            lxc_state_file.write(current_status)

        ### If container is not launch, launch it
        if requested_status != current_status:
            logger.debug('Start lxc.')
            lxc_start = os.path.join(sr_directory,
                                     'parts/lxc/bin/lxc-start')
            call([lxc_start, '-f', lxc_conf_filename,
                  '-n', name,
                  '-d'])

        ################################
        # Stateless shellinabox launch #
        ################################

        current_status = 'stopped'
        # Check if shellinabox is started
        if name in database:
            pid = int(database[name])
            if is_pid_running(pid):
                current_status = 'started'
        with open(sib_state_path, 'w') as sib_state_file:
            sib_state_file.write(current_status)

        if current_status == 'stopped':
            logger.debug('Start shellinabox.')
            shellinabox_pid = call_daemonize([os.path.join(partition_path,
                                                           'bin/shellinaboxd')])
            database[name] = str(shellinabox_pid)



def is_pid_running(pid):
    logger = logging.getLogger('pid')
    logger.debug('Check if pid %d is running.', pid)
    # XXX: Magic number 0 for no special signal
    try:
        os.kill(pid, 0)
        logger.debug('%d is running.', pid)
        return True
    except OSError:
        # Process doesn't exists
        logger.debug('%d is not running.', pid)
        return False

def call(command_line, override_environ={}):
    logger = logging.getLogger('commandline')
    logger.debug('Call %s', ' '.join(command_line))

    environ = dict(os.environ)
    environ.update(override_environ)
    process = subprocess.Popen(command_line, stdin=subprocess.PIPE,
                               stdout=subprocess.PIPE, env=environ)
    process.stdin.flush()
    process.stdin.close()

    if process.wait() != 0:
        logger.debug('Failed')
        raise SlapContainerError("Subprocess call failed")

    out = process.stdout.read()
    logger.debug('Output : %s.', out)
    return out



def call_daemonize(command_line, override_environ={}):
    logger = logging.getLogger('daemon')

    environ = dict(os.environ)
    environ.update(override_environ)

    daemon = subprocess.Popen(command_line, env=environ)
    logger.debug('Daemonize as pid %d : %s', daemon.pid,
                                             ' '.join(command_line))
    return daemon.pid



from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise, TestResult
import re
import time
import os

@implementer(interface.IPromise)
class RunPromise(GenericPromise):
  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    # set periodicity to run the promise twice per day
    self.custom_frequency = int(self.getConfig('frequency', 720))
    self.setPeriodicity(self.custom_frequency)
    # Skip test check on this promise
    self.setTestLess()

  def sense(self):
    """
      Check if http log file contain errors
    """
    log_file = self.getConfig('log-file')
    maximum_delay = int(self.getConfig('maximum-delay', 0))
    if not log_file:
      raise ValueError("log file was not set in promise parameters.")

    regex = re.compile(br"^(\[[^\]]+\]) (\[[^\]]+\]) (.*)$")
    error_amount = 0
    no_route_error = 0
    network_is_unreachable = 0
    timeout = 0
    parsing_failure = 0

    if not os.path.exists(log_file):
      # file don't exist, nothing to check
      self.logger.info("OK")
      return

    with open(log_file, "rb") as f:
      f.seek(0, 2)
      block_end_byte = f.tell()
      f.seek(-min(block_end_byte, 4096), 1)
      data = f.read()
      for line in reversed(data.splitlines()):
        m = regex.match(line)
        if m is None:
          continue
        dt, level, msg = m.groups()
        try:
          try:
            t = time.strptime(dt[1:-1].decode('utf-8'), "%a %b %d %H:%M:%S %Y")
          except ValueError:
            # Fail to parser for the first time, try a different output.
            t = time.strptime(dt[1:-1].decode('utf-8'), "%a %b %d %H:%M:%S.%f %Y")
        except ValueError:
            # Probably it fail to parse
            if parsing_failure < 3:
              # Accept failure 2 times, as the line can be actually
              # cut on the middle.
              parsing_failure += 1
              continue
            raise
        if maximum_delay and (time.time()-time.mktime(t)) > maximum_delay:
          # no result in the latest hour
          break
        if level != b"[error]":
          continue
        # Classify the types of errors
        if b"(113)No route to host" in msg:
          no_route_error += 1
        elif b"(101)Network is unreachable" in msg:
          network_is_unreachable += 1
        elif b"(110)Connection timed out" in msg:
          timeout += 1
        error_amount += 1
    if error_amount:
      self.logger.error("ERROR=%s (NOROUTE=%s, UNREACHABLENET=%s, TIMEOUT=%s)" % (
        error_amount, no_route_error, network_is_unreachable, timeout))
    else:
      self.logger.info("OK")

  def anomaly(self):
    # only check the result of the two latest sense call
    return self._test(result_count=2, failure_amount=2, latest_minute=self.custom_frequency*3)



# -*- coding: utf-8 -*-
import os, time
import sys
import shutil
import tempfile
import unittest
import json
import pkg_resources

from slapos.monitor import globalstate
from slapos.monitor.runpromise import MonitorPromiseLauncher, getArgumentParser
from slapos.monitor.monitor import Monitoring
from jsonschema import validate

class MonitorGlobalTest(unittest.TestCase):

  monitor_conf = """[monitor]
parameter-file-path = %(base_dir)s/knowledge0.cfg
service-pid-folder = %(base_dir)s/run
private-folder = %(base_dir)s/private
public-folder = %(base_dir)s/public
public-path-list = %(public_path_list)s
private-path-list = %(private_path_list)s
crond-folder = %(base_dir)s/cron.d
logrotate-folder = %(base_dir)s/logrotate.d
root-title = %(root_title)s
pid-file =  %(base_dir)s/monitor.pid
log-folder = %(base_dir)s/log
log-backup-folder = %(base_dir)s/log-backup
document-folder = %(base_dir)s/private/documents
parameter-list = 
  raw monitor-user admin
  file sample %(base_dir)s/param
  htpasswd monitor-password %(base_dir)s/.monitor_pwd admin %(base_dir)s/monitor-htpasswd
  httpdcors cors-domain %(base_dir)s/test-httpd-cors.cfg /bin/echo

webdav-folder = %(base_dir)s/webdav
collect-script = %(collect_run_script)s
statistic-script = %(statistic_script)s
python = python
monitor-url-list = %(url_list)s
collector-db = 
base-url = %(base_url)s
title = %(title)s
promise-output-file = %(base_dir)s/monitor-bootstrap-status
promise-runner = %(promise_run_script)s
randomsleep = /bin/echo sleep

[promises]
output-folder = %(base_dir)s/public
legacy-promise-folder = %(etc_dir)s/promise
promise-folder = %(etc_dir)s/plugin
partition-folder = %(base_dir)s
computer-id = COMP-1234
partition-cert = 
partition-key = 
partition-id = slappart0
ipv6 = 2001:34c:1254:df3:89::5df3
software-release = http://some.url.com/software.cfg
master-url = http://10.0.151.118:50000
software-type = default
ipv4 = 10.0.151.118
"""

  def setUp(self):
    self.base_dir = tempfile.mkdtemp()
    self.etc_dir = os.path.join(self.base_dir, 'etc')
    self.output_dir = os.path.join(self.base_dir, '.slapgrid/promise/result')
    os.mkdir(self.etc_dir)
    os.mkdir(os.path.join(self.etc_dir, 'plugin'))
    os.mkdir(os.path.join(self.etc_dir, 'promise'))
    os.mkdir(os.path.join(self.base_dir, 'public'))
    os.mkdir(os.path.join(self.base_dir, 'private'))
    os.mkdir(os.path.join(self.base_dir, 'cron.d'))
    os.mkdir(os.path.join(self.base_dir, 'log'))
    os.mkdir(os.path.join(self.base_dir, 'log-backup'))
    os.mkdir(os.path.join(self.base_dir, 'logrotate.d'))
    os.mkdir(os.path.join(self.base_dir, 'monitor-report'))
    os.mkdir(os.path.join(self.base_dir, 'webdav'))
    os.mkdir(os.path.join(self.base_dir, 'run'))
    os.mkdir(os.path.join(self.base_dir, 'private/documents'))
    self.writeContent(os.path.join(self.base_dir, 'param'), '12345')
    self.writeContent(os.path.join(self.base_dir, '.monitor_pwd'), 'bcuandjy')
    self.writeContent(os.path.join(self.base_dir, 'test-httpd-cors.cfg'), '')
    self.writeContent(os.path.join(self.base_dir, 'monitor-htpasswd'), '12345')

    self.monitor_config_file = os.path.join(self.base_dir, 'monitor.conf')
    self.public_dir = os.path.join(self.base_dir, 'public')
    self.private_dir = os.path.join(self.base_dir, 'private')

    monitor_schema_string = \
      pkg_resources.resource_string(
        'slapos.monitor',
        'doc/monitor_instance.schema.json')
    self.monitor_instance_schema = json.loads(monitor_schema_string.decode('utf-8'))


    self.monitor_config_dict = dict(
      monitor_conf=self.monitor_config_file,
      base_dir=self.base_dir,
      root_title="Monitor ROOT",
      title="Monitor",
      url_list="https://sub.monitor.test.com https://sub2.monitor.test.com",
      base_url="https://monitor.test.com",
      etc_dir=self.etc_dir,
      promise_runner_pid=os.path.join(self.base_dir, 'run', 'monitor-promises.pid'),
      public_folder=os.path.join(self.base_dir, 'public'),
      public_path_list="",
      private_path_list="",
      promise_run_script="/bin/echo",
      collect_run_script="/bin/echo",
      statistic_script="/bin/echo"
    )

  def tearDown(self):
    if os.path.exists(self.base_dir):
      shutil.rmtree(self.base_dir)

  def writeContent(self, file_path, config):
    with open(file_path, 'w') as cfg:
      cfg.write(config)

  def writePromise(self, name, success=True):
    if success:
      result_dict = {'output': 'success', 'code': 0}
    else:
      result_dict = {'output': 'error', 'code': 1}
    content = """#!/bin/sh

echo "%(output)s"
exit %(code)s
""" % result_dict
    promise_path = os.path.join(self.etc_dir, 'promise', name)
    self.writeContent(promise_path, content)
    os.chmod(promise_path, 0o755)
    return promise_path

  def getPromiseParser(self):
    pid_path = os.path.join(self.base_dir, 'run', 'monitor-promise.pid')

    promise_cmd = [
      '--pid-path',
      '%s' % pid_path, '-c', self.monitor_config_file]
    arg_parser = getArgumentParser()
    return arg_parser.parse_args(promise_cmd)

  def test_monitor_instance_state(self):
    self.maxDiff = None
    config_content = self.monitor_conf % self.monitor_config_dict
    self.writeContent(self.monitor_config_file, config_content)

    instance = Monitoring(self.monitor_config_file)
    instance.bootstrapMonitor()

    self.writePromise('promise_1')
    self.writePromise('promise_2', success=False)
    self.writePromise('promise_3', success=False)
    self.writePromise('promise_4')
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    self.assertTrue(os.path.exists(os.path.join(self.output_dir, 'promise_1.status.json')))
    self.assertTrue(os.path.exists(os.path.join(self.output_dir, 'promise_2.status.json')))
    self.assertTrue(os.path.exists(os.path.join(self.output_dir, 'promise_3.status.json')))
    self.assertTrue(os.path.exists(os.path.join(self.output_dir, 'promise_4.status.json')))

    os.symlink(self.output_dir, '%s/public/promise' % self.base_dir)
    # create files for cleanup
    must_stay_public = os.path.join(self.public_dir, 'must_stay')
    must_unlink_public = os.path.join(self.public_dir, 'must_unlink.history.json')
    for f in [must_stay_public, must_unlink_public]:
      with open(f, 'w') as fh:
        fh.write('data')
    # generate instance state files
    document_list_file = os.path.join(self.public_dir, '_document_list')
    with open(document_list_file, 'a+') as fh:
      fh.write('must_unlink.history')

    globalstate.run(self.monitor_config_file)
    with open(document_list_file) as fh:
      self.assertNotIn('must_unlink.history', fh.read())
    self.assertTrue(os.path.exists(must_stay_public))
    self.assertFalse(os.path.exists(must_unlink_public))
    self.assertTrue(os.path.exists(os.path.join(self.public_dir, 'feed')))
    self.assertTrue(os.path.exists(os.path.join(self.public_dir, 'monitor.global.json')))
    self.assertTrue(os.path.exists(os.path.join(self.private_dir, 'monitor.global.json')))
    expected_result = """{
	"status": "ERROR",
	"partition_id": "slappart0",
	"ipv6": "2001:34c:1254:df3:89::5df3",
	"ipv4": "10.0.151.118",
	"software_release": "http://some.url.com/software.cfg",
	"software_type": "default",
	"parameters": [{
		"key": "",
		"value": "admin",
		"title": "monitor-user"
	}, {
		"key": "sample",
		"value": "12345",
		"title": "sample"
	}, {
		"key": "monitor-password",
		"value": "bcuandjy",
		"title": "monitor-password"
	}, {
		"key": "cors-domain",
		"value": "",
		"title": "cors-domain"
	}],
	"title": "Monitor",
	"data": {
		"process_state": "monitor_process_resource.status",
		"io_resource": "monitor_resource_io.data",
		"state": "monitor_state.data",
		"memory_resource": "monitor_resource_memory.data",
		"process_resource": "monitor_resource_process.data",
		"monitor_process_state": "monitor_resource.status"
	},
	"portal_type": "Software Instance",
	"state": {
		"success": 2,
		"error": 2
	},
	"_links": {
		"rss_url": {
			"href": "https://monitor.test.com/public/feed"
		},
		"public_url": {
			"href": "https://monitor.test.com/share/public/"
		},
		"private_url": {
			"href": "https://monitor.test.com/share/private/"
		},
		"related_monitor": [
		  {"href": "https://sub.monitor.test.com/share/public"},
      {"href": "https://sub2.monitor.test.com/share/public"}
    ]
	},
	"aggregate_reference": "COMP-1234",
	"type": "global",
	"specialise_title": "Monitor ROOT"
}"""

    with open(os.path.join(self.private_dir, 'monitor.global.json')) as r:
      result = json.load(r)
      result.pop("date")
      self.assertEqual(result,
        json.loads(expected_result))

    # all promises are OK now
    self.writePromise('promise_2', success=True)
    self.writePromise('promise_3', success=True)
    # rerun right now
    promise_runner.config.force = True
    promise_runner.start()
    globalstate.run(self.monitor_config_file)

    expected_result_dict = json.loads(expected_result)
    expected_result_dict["status"] = "OK"
    expected_result_dict["state"] = {'error': 0, 'success': 4}
    instance_result_dict = None
    with open(os.path.join(self.private_dir, 'monitor.global.json')) as r:
      instance_result_dict = json.load(r)
      result = instance_result_dict.copy()
      result.pop("date")
      self.assertEqual(result,
        expected_result_dict)

    # validate returned json result
    validate(instance_result_dict, self.monitor_instance_schema)


class MonitorGlobalTestWithoutLegacyPromiseFolder(MonitorGlobalTest):
  monitor_conf = """[monitor]
parameter-file-path = %(base_dir)s/knowledge0.cfg
service-pid-folder = %(base_dir)s/run
private-folder = %(base_dir)s/private
public-folder = %(base_dir)s/public
public-path-list = %(public_path_list)s
private-path-list = %(private_path_list)s
crond-folder = %(base_dir)s/cron.d
logrotate-folder = %(base_dir)s/logrotate.d
root-title = %(root_title)s
pid-file =  %(base_dir)s/monitor.pid
log-folder = %(base_dir)s/log
log-backup-folder = %(base_dir)s/log-backup
document-folder = %(base_dir)s/private/documents
parameter-list = 
  raw monitor-user admin
  file sample %(base_dir)s/param
  htpasswd monitor-password %(base_dir)s/.monitor_pwd admin %(base_dir)s/monitor-htpasswd
  httpdcors cors-domain %(base_dir)s/test-httpd-cors.cfg /bin/echo

webdav-folder = %(base_dir)s/webdav
collect-script = %(collect_run_script)s
statistic-script = %(statistic_script)s
python = python
monitor-url-list = %(url_list)s
collector-db = 
base-url = %(base_url)s
title = %(title)s
promise-output-file = %(base_dir)s/monitor-bootstrap-status
promise-runner = %(promise_run_script)s
randomsleep = /bin/echo sleep

[promises]
output-folder = %(base_dir)s/public
promise-folder = %(etc_dir)s/plugin
partition-folder = %(base_dir)s
computer-id = COMP-1234
partition-cert = 
partition-key = 
partition-id = slappart0
ipv6 = 2001:34c:1254:df3:89::5df3
software-release = http://some.url.com/software.cfg
master-url = http://10.0.151.118:50000
software-type = default
ipv4 = 10.0.151.118
"""



##############################################################################
#
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.test.promise.plugin import TestPromisePluginMixin
from slapos.grid.promise import PromiseError
import os

class TestCheckRe6stOptimalStatus(TestPromisePluginMixin):

  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    self.promise_name = "check-icmp-packet-lost.py"

    self.base_content = """from slapos.promise.plugin.check_re6st_optimal_status import RunPromise

extra_config_dict = {
  'ipv4': '%(ipv4)s',
  'count': '%(count)s',
  'ipv6': '%(ipv6)s',
}
"""

  def tearDown(self):
    TestPromisePluginMixin.tearDown(self)

  def test_ipv6_is_faster(self):
    content = self.base_content % {
      'ipv4': "8.8.8.8",
      'ipv6': '::1',
      'count': 5,
    }
    self.writePromise(self.promise_name, content)

    self.configureLauncher(force=True, timeout=20, enable_anomaly=True)
    self.launcher.run()
    # run a second time to add more results
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    last_message = result['result']['message'].split('\n')[-1]
    self.assertEqual(result['result']['failed'], False)
    #self.assertEqual(last_message, "OK: IPv4 reachable, IPv6 reachable")
    # some testnodes cannot ping because they are qemu VM with Nat network, and
    # ICMP is disabled. Expected result is "OK: IPv4 reachable, IPv6 reachable"
    # but it ICMP is not working, we will have "IPv4 unreachable" in the message
    # so we will test only if the ping returned "OK" and if IPv6 was reachable.
    self.assertTrue(last_message.startswith("OK:"))
    self.assertTrue("IPv6 reachable" in last_message)

  def test_ipv4_is_faster(self):
    content = self.base_content % {
      'ipv4': "127.0.0.1",
      'ipv6': '2001:67c:1254::1',
      'count': 5,
    }
    self.writePromise(self.promise_name, content)

    self.configureLauncher(force=True, timeout=20, enable_anomaly=True)
    self.launcher.run()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    last_message = result['result']['message'].split('\n')[-1]
    self.assertEqual(result['result']['failed'], True)
    self.assertTrue(last_message.startswith('FAIL'))

  def test_ipv4_unreacheable_and_ipv6_ok(self):
    content = self.base_content % {
      'ipv4': "couscous",
      'ipv6': '::1',
      'count': 5,
    }
    self.writePromise(self.promise_name, content)

    self.configureLauncher(force=True, timeout=20, enable_anomaly=True)
    self.launcher.run()
    # run a second time to add more results
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    last_message = result['result']['message'].split('\n')[-1]
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(last_message, "OK: IPv4 unreachable, IPv6 reachable")

  def test_ipv6_fail(self):
    content = self.base_content % {
      'ipv4': "127.0.0.1",
      'ipv6': 'couscous',
      'count': 5,
    }
    self.writePromise(self.promise_name, content)

    self.configureLauncher(force=True, timeout=20, enable_anomaly=True)
    self.launcher.run()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    last_message = result['result']['message'].split('\n')[-1]
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(last_message, "FAILED: IPv4 reachable, IPv6 unreachable")

if __name__ == '__main__':
  unittest.main()



from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise

try:
  import subprocess32 as subprocess
except ImportError:
  import subprocess

@implementer(interface.IPromise)
class RunPromise(GenericPromise):
  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    self.setPeriodicity(minute=int(self.getConfig('frequency', 5)))

  def sense(self):
    """
      Check trafficserver cache availability
    """
    wrapper = self.getConfig('wrapper-path')

    if 'traffic_line' in wrapper:
      args = [wrapper, '-r',  'proxy.node.cache.percent_free']
      message = "Cache not available, availability: %s"
    elif 'traffic_ctl' in wrapper:
      args = [wrapper, 'metric', 'get', 'proxy.process.cache.percent_full']
      message = "Cache not available, occupation: %s"
    else:
      self.logger.error("Wrapper %r not supported." % (wrapper,))
      return

    try:
      subprocess.check_output(args, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
      result = e.output.strip()
      self.logger.error(message, result if str is bytes else
                                 result.decode('utf-8', 'replace'))
    else:
      self.logger.info("OK")

  def anomaly(self):
    """
      There is an anomaly if last 3 senses were bad.
    """
    return self._anomaly(result_count=3, failure_amount=3)


from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise

import socket

ADDRESS_USAGE = (
  "Address must be specified in 1 of the following 3 forms:"
  " (host, port), path or abstract")

@implementer(interface.IPromise)
class RunPromise(GenericPromise):
  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    self.setPeriodicity(minute=2)
    self.result_count = int(self.getConfig('result-count', 3))
    self.failure_amount = int(self.getConfig('failure-amount', 3))


  def sense(self):
    """
      Check the state of a socket.
    """
    host = self.getConfig('host')
    port = self.getConfig('port')
    path = self.getConfig('pathname')
    abstract = self.getConfig('abstract')

    if host:
      if path or abstract or not port:
        self.logger.error(ADDRESS_USAGE)
        return
      # type of port must be int or str, unicode is not accepted.
      family, _, _, _, addr = socket.getaddrinfo(host, int(port))[0]
    else:
      if bool(path) == bool(abstract):
        self.logger.error(ADDRESS_USAGE)
        return
      family = socket.AF_UNIX
      addr = path or '\0' + abstract

    s = socket.socket(family, socket.SOCK_STREAM)
    try:
      s.connect(addr)
    except socket.error as e:
      self.logger.error('%s: %s', type(e).__name__, e)
    else:
      self.logger.info("socket connection OK %r", addr)
    finally:
      s.close()


  def anomaly(self):
    """
      By default, there is an anomaly if last 3 senses were bad.
    """
    return self._anomaly(result_count=self.result_count, failure_amount=self.failure_amount)



##############################################################################
#
# Copyright (c) 2019 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.grid.promise import PromiseError
from slapos.test.promise.plugin import TestPromisePluginMixin

import tempfile
import os
import unittest
import shutil
import six


class TestCheckFileState(TestPromisePluginMixin):
  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    self.tempdir = tempfile.mkdtemp()
    self.promise_name = "check-file-state.py"

    self.base_content = """from slapos.promise.plugin.check_file_state import RunPromise

extra_config_dict = {
  'filename': %(filename)r,
  'state': %(state)r,
  'url': %(url)r,
}
"""

  def tearDown(self):
    TestPromisePluginMixin.tearDown(self)
    shutil.rmtree(self.tempdir)

  def test_check_file_directory(self):
    filename = os.path.join(self.tempdir, 'test.file')
    os.mkdir(filename)
    content = self.base_content % {
      'filename': filename,
      'state': 'not-empty',
      'url': 'https://www.example.com/',
    }
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR %s(21, 'Is a directory') "
      "during opening and reading file %r" % (
        "IsADirectoryError" if six.PY3 else "IOError",
        filename)
    )

  def test_check_file_not_exists(self):
    filename = os.path.join(self.tempdir, 'test.file')
    content = self.base_content % {
      'filename': filename,
      'state': 'not-empty',
      'url': 'https://www.example.com/',
    }
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR %s(2, 'No such file or directory') "
      "during opening and reading file %r" % (
        "FileNotFoundError" if six.PY3 else "IOError",
        filename)
    )

  def test_check_file_empty(self):
    filename = os.path.join(self.tempdir, 'test.file')
    with open(filename, 'w') as fh:
      fh.write('')
    content = self.base_content % {
      'filename': filename,
      'state': 'empty',
      'url': 'https://www.example.com/',
    }
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      "OK %r state 'empty'" % (filename,)
    )

  def test_check_file_not_empty_fail(self):
    filename = os.path.join(self.tempdir, 'test.file')
    with open(filename, 'w') as fh:
      fh.write('')
    content = self.base_content % {
      'filename': filename,
      'state': 'not-empty',
      'url': 'https://www.example.com/',
    }
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR %r empty" % (filename,)
    )

  def test_check_file_not_empty(self):
    filename = os.path.join(self.tempdir, 'test.file')
    with open(filename, 'w') as fh:
      fh.write('content')
    content = self.base_content % {
      'filename': filename,
      'state': 'not-empty',
      'url': 'https://www.example.com/',
    }
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      "OK %r state 'not-empty'" % (filename,)
    )

  def test_check_file_empty_fail(self):
    filename = os.path.join(self.tempdir, 'test.file')
    with open(filename, 'w') as fh:
      fh.write('content')
    content = self.base_content % {
      'filename': filename,
      'state': 'empty',
      'url': 'https://www.example.com/',
    }
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR %r not empty, content available at "
      "https://www.example.com/" % (filename,)
    )


if __name__ == '__main__':
  unittest.main()
